/* ---------------------------------------------------------------------
*
*  -- PBLAS routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*  ---------------------------------------------------------------------
*/
/*
*  Include files
*/
#include "pblas.h"

void pdsyr2_( uplo, n, alpha, X, ix, jx, desc_X, incx, Y, iy, jy,
              desc_Y, incy, A, ia, ja, desc_A )
/*
*  .. Scalar Arguments ..
*/
   F_CHAR      uplo;
   int         * ia, * incx, * incy, * ix, *iy, * ja, * jx, * jy, * n;
   double      * alpha;
/* ..
*  .. Array Arguments ..
*/
   int         desc_A[], desc_X[], desc_Y[];
   double      A[], X[], Y[];
{
/*
*  Purpose
*  =======
*
*  PDSYR2  performs the symmetric rank 2 operation
*
*     sub( A ) := alpha*sub( X )*sub( Y )' +
*                 alpha*sub( Y )*sub( X )' + sub( A ) ,
*
*  where sub( A ) denotes A(IA:IA+N-1,JA:JA+N-1),
*
*  where sub( X ) denotes X(IX,JX:JX+N-1) if INCX = M_X,
*                         X(IX:IX+N-1,JX) if INCX = 1 and INCX <> M_X,
*
*        sub( Y ) denotes Y(IY,JY:JY+N-1) if INCY = M_Y,
*                         Y(IY:IY+N-1,JY) if INCY = 1 and INCY <> M_Y,
*
*  alpha is a scalar, sub( X ) and sub( Y ) are N element distributed
*  vectors and sub( A ) is an N-by-N symmetric distributed matrix.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESC_A:
*
*  NOTATION        STORED IN     EXPLANATION
*  --------------- ------------- ---------------------------------------
*  M_A    (global) desc_A[M_]    The number of rows in the distributed
*                                matrix.
*  N_A    (global) desc_A[N_]    The number of columns in the distribu-
*                                ted matrix.
*  MB_A   (global) desc_A[MB_]   The blocking factor used to distribute
*                                the rows of the matrix.
*  NB_A   (global) desc_A[NB_]   The blocking factor used to distribute
*                                the columns of the matrix.
*  RSRC_A (global) desc_A[RSRC_] The process row over which the first
*                                row of the matrix is distributed.
*  CSRC_A (global) desc_A[CSRC_] The process column over which the first
*                                column of the matrix is distributed.
*  CTXT_A (global) desc_A[CTXT_] The BLACS context handle, indicating
*                                the global context of the operation on
*                                the matrix.
*                                The context itself is global.
*  LLD_A  (local)  desc_A[LLD_]  The leading dimension of the local
*                                array storing the local blocks of the
*                                distributed matrix A.
*                                LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC.
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Because vectors may be seen as particular matrices, a distributed
*  vector is considered to be a distributed matrix.
*
*  sub( X ) and sub( Y ) must be distributed along the same axis, i.e
*  if INCX = M_X (resp. (INCX = 1 and INCX <> M_X)), it is required
*  that INCY = M_Y (resp. (INCY = 1 and INCY <> M_Y)).  Moreover, the
*  restrictions on sub( X ) and sub( Y ) are the same, only the ones
*  on sub( X ) are detailed:
*
*  If INCX = M_X, the process column having the first entries of
*  sub( X ) must also contain the first block of sub( A ), similarly
*  if INCX = 1 and INCX <> M_X, the process row having the first
*  entries of sub( X ) must also contain the first block of sub( A ).
*  It is also required that MOD(IA-1,MB_A) = MOD(JA-1,NB_A) and
*   MB_A = NB_A.
*
*  If INCX = M_X (resp. (INCX = 1 and INCX <> M_X)), the column (resp.
*  row) offset of sub( X ) must be equal to the row offset of sub( A ),
*  i.e MOD(JX-1,MB_X) (resp. MOD(IX-1,NB_X)) = MOD(IA-1,MB_A). Moreover,
*  the column (resp. row) blocksize of X must be equal to the column
*  blocksize of A, i.e NB_X = NB_A (resp. MB_X = NB_A).
*
*  Parameters
*  ==========
*
*  UPLO    (global input) pointer to CHARACTER
*          On entry, UPLO specifies whether the upper or lower
*          triangular part of the distributed matrix sub( A ) is to be
*          referenced as follows:
*
*             UPLO = 'U' or 'u'   Only the upper triangular part of
*                                 sub( A ) is to be referenced.
*
*             UPLO = 'L' or 'l'   Only the lower triangular part of
*                                 sub( A ) is to be referenced.
*
*  N       (global input) pointer to INTEGER
*          The order of the distributed matrix sub( A ). N >= 0.
*
*  ALPHA   (global input) pointer to DOUBLE PRECISION
*          On entry, ALPHA specifies the scalar alpha.
*
*  X       (local input) DOUBLE PRECISION array containing the local
*          pieces of a distributed matrix of dimension of at least
*                  ( (JX-1)*M_X + IX + ( N - 1 )*abs( INCX ) )
*          This array contains the entries of the distributed vector
*          sub( X ).
*
*  IX      (global input) pointer to INTEGER
*          The global row index of the submatrix of the distributed
*          matrix X to operate on.
*
*  JX      (global input) pointer to INTEGER
*          The global column index of the submatrix of the distributed
*          matrix X to operate on.
*
*  DESCX   (global and local input) INTEGER array of dimension 8.
*          The array descriptor of the distributed matrix X.
*
*  INCX    (global input) pointer to INTEGER
*          The global increment for the elements of X. Only two values
*          of INCX are supported in this version, namely 1 and M_X.
*
*  Y       (local input) DOUBLE PRECISION array containing the local
*          pieces of a distributed matrix of dimension of at least
*                  ( (JY-1)*M_Y + IY + ( N - 1 )*abs( INCY ) )
*          This array contains the entries of the distributed vector
*          sub( Y ).
*
*  IY      (global input) pointer to INTEGER
*          The global row index of the submatrix of the distributed
*          matrix Y to operate on.
*
*  JY      (global input) pointer to INTEGER
*          The global column index of the submatrix of the distributed
*          matrix Y to operate on.
*
*  DESCY   (global and local input) INTEGER array of dimension 8.
*          The array descriptor of the distributed matrix Y.
*
*  INCY    (global input) pointer to INTEGER
*          The global increment for the elements of Y. Only two values
*          of INCY are supported in this version, namely 1 and M_Y.
*
*  A       (local input/local output) DOUBLE PRECISION pointer into the
*          local memory to an array of dimension (LLD_A,LOCq(JA+N-1).
*          Before entry with  UPLO = 'U' or 'u', the leading N-by-N
*          upper triangular part of the distributed matrix sub( A )
*          must contain the upper triangular part of the symmetric
*          distributed matrix and the strictly lower triangular part
*          of sub( A ) is not referenced. On exit, the upper triangular
*          part of sub( A ) is overwritten by the upper triangular
*          part of the updated distributed matrix.
*          Before entry with UPLO = 'L' or 'l', the leading N-by-N
*          lower triangular part of the distributed matrix sub( A )
*          must contain the lower triangular part of the symmetric
*          distributed matrix and the strictly upper triangular part
*          of sub( A ) is not referenced. On exit, the lower triangular
*          part of sub( A ) is overwritten by the lower triangular
*          part of the updated distributed matrix.
*
*  IA      (global input) pointer to INTEGER
*          The global row index of the submatrix of the distributed
*          matrix A to operate on.
*
*  JA      (global input) pointer to INTEGER
*          The global column index of the submatrix of the distributed
*          matrix A to operate on.
*
*  DESCA   (global and local input) INTEGER array of dimension 8.
*          The array descriptor of the distributed matrix A.
*
*  =====================================================================
*
*  .. Local Scalars ..
*/
   char        * top, UploA, xydist;
   int         iacol, iarow, icoffa, ictxt, iia, iix, iiy, info, iroffa,
               iszcmp, ixcol, ixpos, ixrow, iycol, iypos, iyrow, jja,
               jjx, jjy, lcm, lcmp, lcmq, locincx, locincy, mullen,
               mycol, myrow, nca, nn, np0, nprow, npcol, nra, nq0,
               szcmp, tmp1, tmp2, wksz;
/* ..
*  .. PBLAS Buffer ..
*/
   double      * buff;
/* ..
*  .. External Functions ..
*/
   void        blacs_gridinfo_();
   void        pbchkmat();
   void        pbchkvect();
   void        pberror_();
   char        * getpbbuf();
   char        * ptop();
   F_VOID_FCT  pbdsyr2_();
   F_INTG_FCT  ilcm_();
/* ..
*  .. Executable Statements ..
*
*  Get grid parameters
*/
   ictxt = desc_X[CTXT_];
   blacs_gridinfo_( &ictxt, &nprow, &npcol, &myrow, &mycol );
/*
*  Test the input parameters
*/
   info = 0;
   if( nprow == -1 )
      info = -707;
   else
   {
      UploA = Mupcase( F2C_CHAR( uplo )[0] );
      iroffa = (*ia-1) % desc_A[MB_];
      icoffa = (*ja-1) % desc_A[NB_];
      pbchkvect( *n, 2, *ix, *jx, desc_X, *incx, 7, &iix, &jjx,
                 &ixrow, &ixcol, nprow, npcol, myrow, mycol, &info );
      pbchkvect( *n, 2, *iy, *jy, desc_Y, *incy, 12, &iiy, &jjy,
                 &iyrow, &iycol, nprow, npcol, myrow, mycol, &info );
      pbchkmat( *n, 2, *n, 2, *ia, *ja, desc_A, 17, &iia, &jja,
                &iarow, &iacol, nprow, npcol, myrow, mycol,
                &nra, &nca, &info );
      if( info == 0 )
      {
         if( ( UploA != 'U' ) && ( UploA != 'L' ) )
            info = -1;
         else if( iroffa != icoffa )
            info = -16;
         if( ( *incx == desc_X[M_] ) && ( *incy == desc_Y[M_] ) )
         {
            if( ( ( (*jx-1) % desc_X[NB_] ) != iroffa ) ||
                ( ( (*jy-1) % desc_Y[NB_] ) != iroffa ) )
               info = -15;
            else if( ( ( (*jx-1) % desc_X[NB_] ) != icoffa ) ||
                ( ( (*jy-1) % desc_Y[NB_] ) != icoffa ) ||
                ( ixcol != iacol ) ||
                ( iycol != iacol ) )
               info = -16;
            else if( desc_A[MB_] != desc_X[NB_] )
               info = -1703;
            else if( ( desc_A[MB_] != desc_A[NB_] ) ||
                     ( desc_A[NB_] != desc_Y[NB_] ) )
               info = -1704;
         }
         else if( ( *incx == 1 ) && ( *incx != desc_X[M_] ) &&
                  ( *incy == 1 ) && ( *incy != desc_Y[M_] ) )
         {
            if( ( ( (*ix-1) % desc_X[MB_] ) != iroffa ) ||
                ( ( (*iy-1) % desc_Y[MB_] ) != iroffa ) ||
                ( ixrow != iarow ) ||
                ( iyrow != iarow ) )
               info = -15;
            else if( ( ( (*ix-1) % desc_X[MB_] ) != icoffa ) ||
                ( ( (*iy-1) % desc_Y[MB_] ) != icoffa ) )
               info = -16;
            else if( desc_A[MB_] != desc_X[MB_] )
               info = -1703;
            else if( ( desc_A[MB_] != desc_A[NB_] ) ||
                     ( desc_A[NB_] != desc_Y[MB_] ) )
               info = -1704;
         }
         else
         {
            if( ( *incx == 1 ) && ( *incx != desc_X[M_] ) )
               info = -8;
            else if( ( *incy == 1 ) && ( *incy != desc_Y[M_] ) )
               info = -13;
         }
         if( ictxt != desc_Y[CTXT_] )
            info = -1207;
         if( ictxt != desc_A[CTXT_] )
            info = -1707;
      }
   }
   if( info )
   {
      pberror_( &ictxt, "PDSYR2", &info );
      return;
   }
/*
*  Quick return if possible.
*/
   if( (*n == 0) || ( *alpha == ZERO ) )
      return;
/*
*  Figure out the arguments to be passed to pbdsyr2
*/
   nn = *n + iroffa;
   mullen = MULLENFAC * desc_A[MB_];
   lcm = ilcm_( &nprow, &npcol );

   if( ( *incx == desc_X[M_] ) && ( *incy == desc_Y[M_] ) )
   {
      lcmp = lcm / nprow;
      lcmq = lcm / npcol;
      top = ptop( BROADCAST, COLUMN, TOPGET );
      xydist = 'R';
      locincx = desc_X[LLD_];
      locincy = desc_Y[LLD_];
      ixpos = ixrow; iypos = iyrow;
      tmp1 = nn / desc_A[MB_];
      np0 = MYROC0( tmp1, nn, desc_A[MB_], nprow );
      tmp2 = nn / desc_A[NB_];
      nq0 = MYROC0( tmp2, nn, desc_A[NB_], npcol );
      tmp2 = lcmq * desc_A[MB_];
      iszcmp = CEIL( mullen, tmp2 );
      szcmp = iszcmp * iszcmp * tmp2 * lcmp*desc_A[MB_];
      tmp1 = np0 / desc_A[MB_];
      tmp2 = CEIL( nn, desc_A[MB_] );
      tmp1 = MYROC0( tmp1, np0, desc_A[MB_], lcmp ) * MIN( lcmp, tmp2 );
      wksz = np0 + 2*nq0 + MAX( szcmp, tmp1 );
   }
   else
   {
      lcmp = lcm / nprow;
      lcmq = lcm / npcol;
      top = ptop( BROADCAST, ROW, TOPGET );
      xydist = 'C';
      locincx = 1;
      locincy = 1;
      ixpos = ixcol;
      iypos = iycol;
      tmp1 = nn / desc_A[MB_];
      np0 = MYROC0( tmp1, nn, desc_A[MB_], nprow );
      tmp2 = nn / desc_A[NB_];
      nq0 = MYROC0( tmp2, nn, desc_A[NB_], npcol );
      tmp2 = lcmq * desc_A[MB_];
      iszcmp = CEIL( mullen, tmp2 );
      szcmp = iszcmp * iszcmp * tmp2 * lcmp*desc_A[MB_];
      tmp1 = nq0 / desc_A[NB_];
      tmp2 = CEIL( nn, desc_A[NB_] );
      tmp1 = MYROC0( tmp1, nq0, desc_A[NB_], lcmq ) * MIN( lcmq, tmp2 );
      wksz = nq0 + 2*np0 + MAX( szcmp, tmp2 );
   }
   buff = (double *)getpbbuf( "PDSYR2", wksz*sizeof(double) );
/*
*  Call PB-BLAS routine
*/
   pbdsyr2_( &ictxt, uplo, C2F_CHAR( &xydist ), n, &desc_A[MB_],
             &iroffa, alpha, &X[iix-1+(jjx-1)*desc_X[LLD_]], &locincx,
             &Y[iiy-1+(jjy-1)*desc_Y[LLD_]], &locincy,
             &A[iia-1+(jja-1)*desc_A[LLD_]], &desc_A[LLD_],
             &ixpos, &iypos, &iarow, &iacol, C2F_CHAR( top ),
             C2F_CHAR( NO ), C2F_CHAR( NO ), C2F_CHAR( NO ),
             &mullen, buff );
}
