      SUBROUTINE PBZ2CHKINFO( SUMMRY, NOUT, NMAT, MVAL, NVAL, NBMAT,
     $                        MBVAL, NBVAL, MZVAL, NZVAL, NGRIDS, NPVAL,
     $                        NQVAL, ALPHA, BETA, XCOMM, YCOMM, XWORK,
     $                        YWORK, DIAG, BR1ST, MULLEN, IAROW, IACOL,
     $                        IXROW, IXCOL, IYROW, IYCOL, INCX, INCY,
     $                        IASEED, IXSEED, IYSEED, WORK, IAM,
     $                        NNODES )
*
*  -- PB-BLAS auxiliary routine (version 2.0) --
*     Jaeyoung Choi, Oak Ridge National Laboratory
*     Jack Dongarra, University of Tennessee and Oak Ridge National Lab.
*     David Walker,  Oak Ridge National Laboratory
*     February 28, 1995
*
*     ..
*     .. Scalar Arguments ..
      CHARACTER*1             XCOMM, YCOMM, XWORK, YWORK
      CHARACTER*1             DIAG, BR1ST
      INTEGER                 NOUT, NMAT, NBMAT, NGRIDS, MULLEN, IAROW
      INTEGER                 IACOL, IXROW, IXCOL, IYROW, IYCOL, INCX
      INTEGER                 INCY, IASEED, IXSEED, IYSEED, IAM, NNODES
      COMPLEX*16              ALPHA, BETA
*     ..
*     .. Array Arguments ..
      CHARACTER               SUMMRY*( * )
      INTEGER                 WORK( * )
      INTEGER                 MVAL( * ),  NVAL( * )
      INTEGER                 MBVAL( * ), NBVAL( * )
      INTEGER                 MZVAL( * ), NZVAL( * )
      INTEGER                 NPVAL( * ), NQVAL( * )
*     ..
*
*  PURPOSE
*  =======
*
*  PBZ2CHKINFO: Gets needed startup information for PB-BLAS test.
*
*  Arguments
*  =========
*
*  SUMMRY   (output) CHARACTER*40
*           Name of output (summary) file (if any). Only defined for node 0.
*
*  NOUT     (output) INTEGER
*           The unit number for output file. NOUT = 6, ouput to screen,
*           NOUT = 0, output to stderr.  Only defined for node 0.
*
*  NMAT     (output) INTEGER
*           The number of different values that can be used for M, N and K.
*
*  MVAL     (output) INTEGER
*           Array, dimension(LDVAL), the values of M to run code with.
*
*  NVAL     (output) INTEGER
*           Array, dimension(LDVAL), the values of N to run code with.
*
*  NBMAT    (output) INTEGER
*           The number of different values that can be used for MB and NB.
*
*  MBVAL    (output) INTEGER
*           Array, dimension(LDVAL), the values of MB to run code with.
*
*  NBVAL    (output) INTEGER
*           Array, dimension(LDVAL), the values of NB to run code with.
*
*  NGRIDS   (output) INTEGER
*           The number of different values that can be used for P & Q.
*
*  NPVAL    (output) INTEGER
*           Array, dimension(LDPQVAL), the values of P (number of processor
*           rows) to run code with.
*
*  NQVAL    (output) INTEGER
*           Array, dimension(LDPQVAL), the values of Q (number of processor
*           columns) to run code with.
*
*  ALPHA    (output) COMPLEX*16
*           Scalar for the matrix multiply.
*
*  BETA     (output) COMPLEX*16
*           Scalar for the matrix multiply.
*
*  WORK     (workspace) INTEGER dimension (6)
*           Used to pack all input arrays in order to send info in one message.
*           The largest it will ever need to be is
*           LDNVAL+LDNBVAL+LDNRVAL+LDNBRVAL+LDPVAL+LDQVAL.
*
*  IAM      (input) INTEGER
*           My node number.
*
*  NNODES   (input) INTEGER
*           The number of nodes in allocated cube.
*
*=======================================================================
*
*     ..
*     .. Parameters ..
      INTEGER            NIN
      PARAMETER          ( NIN = 11 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, ICONTXT, II
*     ..
*     .. Local Arrays ..
      CHARACTER*79       USRINFO
      COMPLEX*16         TEMP( 2 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GET, BLACS_GRIDEXIT, BLACS_GRIDINIT,
     $                   BLACS_SETUP, IGEBS2D, IGEBR2D, SCOPY, ZGEBS2D,
     $                   ZGEBR2D
*     ..
*     .. External Functions ..
      INTEGER            ICEIL
      EXTERNAL           ICEIL
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          CHAR, ICHAR, MIN, MAX
*     ..
*     .. Executable Statements ..
*
*     Processor 0 reads the input data, broadcasts to other nodes and
*     writes needed information to NOUT
*
      IF( IAM.EQ.0 ) THEN
*
*        Temporarily define blacs grid to include all nodes so
*        information can be broadcast to all nodes
*
*        Open file and skip data file header
*
         OPEN( NIN, FILE='PBZ2BLAS.dat', STATUS='OLD' )
         READ( NIN, * ) SUMMRY
         SUMMRY = ' '
*
*        Read in user-supplied info about machine type, compiler, etc.
*
         READ( NIN, * ) USRINFO
*
*        Read name and unit number for summary output file
*
         READ( NIN, * ) SUMMRY
         READ( NIN, * ) NOUT
         IF( NOUT.NE.0 .AND. NOUT.NE.6 )
     $      OPEN( NOUT, FILE = SUMMRY, STATUS = 'UNKNOWN' )
*
*        Get values of M, N
*
         READ( NIN, * ) NMAT
         READ( NIN, * ) ( MVAL(I), I = 1, NMAT )
         READ( NIN, * ) ( NVAL(I), I = 1, NMAT )
*
*        Get values of MB, NB, MZ, NZ
*
         READ( NIN, * ) NBMAT
         READ( NIN, * ) ( MBVAL(I), I = 1, NBMAT )
         READ( NIN, * ) ( NBVAL(I), I = 1, NBMAT )
         READ( NIN, * ) ( MZVAL(I), I = 1, NBMAT )
         READ( NIN, * ) ( NZVAL(I), I = 1, NBMAT )
*
*        Get values of P, Q
*
         READ( NIN, * ) NGRIDS
         READ( NIN, * ) ( NPVAL(I), I = 1, NGRIDS )
         READ( NIN, * ) ( NQVAL(I), I = 1, NGRIDS )
*
*        Get Alpha, Beta
*
         READ( NIN, * ) ALPHA, BETA
*
*        Get XCOMM, YCOMM
*
         READ( NIN, * ) XCOMM, YCOMM
*
*        Get XWORK, YWORK
*
         READ( NIN, * ) XWORK, YWORK
*
*        Get DIAG, BR1ST
*
         READ( NIN, * ) DIAG, BR1ST
*
*        Get MULLEN
*
         READ( NIN, * ) MULLEN
*
*        Get IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL
*
         READ( NIN, * ) IAROW, IACOL, IXROW, IXCOL, IYROW, IYCOL
*
*        Get INCX, INCY
*
         READ( NIN, * ) INCX, INCY
*
*        Get IASEED, IXSEED, and IYSEED
*
         READ( NIN, * ) IASEED, IXSEED, IYSEED
*
*        Close input file
*
         CLOSE( NIN )
*
*        For pvm only: if virtual machine not set up, allocate it and
*        spawn the correct number of processes.
*
         IF( NNODES.LT.1 ) THEN
             NNODES = 0
             DO 10 I = 1, NGRIDS
                NNODES = MAX( NNODES, NPVAL(I)*NQVAL(I) )
   10        CONTINUE
             CALL BLACS_SETUP( IAM, NNODES )
         END IF
*
*        Temporarily define blacs grid to include all nodes so
*        information can be broadcast to all nodes
*
         CALL BLACS_GET( -1, 0, ICONTXT )
         CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', 1, NNODES )
*
*        Pack information arrays and broadcast
*
         II = 1
         WORK(II) = NMAT
         II = II + 1
         CALL SCOPY( NMAT, MVAL, 1, WORK(II), 1 )
         II = II + NMAT
         CALL SCOPY( NMAT, NVAL, 1, WORK(II), 1 )
         II = II + NMAT
         WORK(II) = NBMAT
         II = II + 1
         CALL SCOPY( NBMAT, MBVAL, 1, WORK(II), 1 )
         II = II + NBMAT
         CALL SCOPY( NBMAT, NBVAL, 1, WORK(II), 1 )
         II = II + NBMAT
         CALL SCOPY( NBMAT, MZVAL, 1, WORK(II), 1 )
         II = II + NBMAT
         CALL SCOPY( NBMAT, NZVAL, 1, WORK(II), 1 )
         II = II + NBMAT
         WORK(II) = NGRIDS
         II = II + 1
         CALL SCOPY( NGRIDS, NPVAL, 1, WORK(II), 1 )
         II = II + NGRIDS
         CALL SCOPY( NGRIDS, NQVAL, 1, WORK(II), 1 )
         II = II + NGRIDS
         WORK(II) = ICHAR( XCOMM )
         II = II + 1
         WORK(II) = ICHAR( YCOMM )
         II = II + 1
         WORK(II) = ICHAR( XWORK )
         II = II + 1
         WORK(II) = ICHAR( YWORK )
         II = II + 1
         WORK(II) = ICHAR( DIAG )
         II = II + 1
         WORK(II) = ICHAR( BR1ST )
         II = II + 1
         WORK(II) = MULLEN
         II = II + 1
         WORK(II) = IAROW
         II = II + 1
         WORK(II) = IACOL
         II = II + 1
         WORK(II) = IXROW
         II = II + 1
         WORK(II) = IXCOL
         II = II + 1
         WORK(II) = IYROW
         II = II + 1
         WORK(II) = IYCOL
         II = II + 1
         WORK(II) = INCX
         II = II + 1
         WORK(II) = INCY
         II = II + 1
         WORK(II) = IASEED
         II = II + 1
         WORK(II) = IXSEED
         II = II + 1
         WORK(II) = IYSEED
         II = II + 1
         CALL IGEBS2D( ICONTXT, 'All', '1-tree', 1, 1, II, 1 )
         CALL IGEBS2D( ICONTXT, 'All', '1-tree', II, 1, WORK, II )
*
         TEMP( 1 ) = ALPHA
         TEMP( 2 ) = BETA
         CALL ZGEBS2D( ICONTXT, 'All', '1-tree', 1, 2, TEMP, 1 )
*
*        regurgitate input
*
         WRITE(NOUT,1000) 'PB-BLAS Level 2 Check routine, '//
     $                    'February 28, 1995, version 2.0'
         WRITE(NOUT,1000) USRINFO
         WRITE(NOUT,1000) '     '
         WRITE(NOUT,1000) 'Running tests of PB-BLAS'
         WRITE(NOUT,1000) 'The matrices A, X and Y are randomly '//
     $                   'generated for each test.'
         WRITE(NOUT,1000) '    '
         WRITE(NOUT,1000) 'An explanation of the input parameters '//
     $                   'as follows:'
         WRITE(NOUT,1000) 'M      : The row number of the matrix'
         WRITE(NOUT,1000) 'N      : The column number of the matrix'
         WRITE(NOUT,1000) '         (N x N is used for square matrices)'
         WRITE(NOUT,1000) 'MB     : The block size along M'
         WRITE(NOUT,1000) 'NB     : The block size along N'
         WRITE(NOUT,1000) 'MZ     : The offset along M'
         WRITE(NOUT,1000) 'NZ     : The offset along N'
         WRITE(NOUT,1000) 'P      : The number of processor rows.'
         WRITE(NOUT,1000) 'Q      : The number of processor columns.'
         WRITE(NOUT,1000) '   '
         WRITE(NOUT,1000) 'The following parameter values will be used:'
         WRITE(NOUT,2000) 'M  : ',( MVAL(I), I = 1, MIN(NMAT, 10) )
         WRITE(NOUT,2000) 'N  : ',( NVAL(I), I = 1, MIN(NMAT, 10) )
         WRITE(NOUT,2000) 'MB : ',( MBVAL(I), I = 1, MIN(NBMAT, 10) )
         WRITE(NOUT,2000) 'NB : ',( NBVAL(I), I = 1, MIN(NBMAT, 10) )
         WRITE(NOUT,2000) 'MZ : ',( MZVAL(I), I = 1, MIN(NBMAT, 10) )
         WRITE(NOUT,2000) 'NZ : ',( NZVAL(I), I = 1, MIN(NBMAT, 10) )
         WRITE(NOUT,2000) 'P  : ',( NPVAL(I), I = 1, MIN(NGRIDS, 10) )
         WRITE(NOUT,2000) 'Q  : ',( NQVAL(I), I = 1, MIN(NGRIDS, 10) )
         WRITE(NOUT,1000)  '    '
         WRITE(NOUT,5000) 'ALPHA, BETA  : ', ALPHA, BETA
         WRITE(NOUT,4000) 'XCOMM, YCOMM : ', XCOMM, YCOMM
         WRITE(NOUT,4000) 'XWORK, YWORK : ', XWORK, YWORK
         WRITE(NOUT,4000) 'DIAG,  BR1ST : ', DIAG, BR1ST
         WRITE(NOUT,3000) 'MULLEN       : ', MULLEN
         WRITE(NOUT,3000) 'IAROW, IACOL : ', IAROW, IACOL
         WRITE(NOUT,3000) 'IXROW, IXCOL : ', IXROW, IXCOL
         WRITE(NOUT,3000) 'IYROW, IYCOL : ', IYROW, IYCOL
         WRITE(NOUT,3000) 'INCX,  INCY  : ', INCX,  INCY
         WRITE(NOUT,3100) 'IASEED, IXSEED, IYSEED : ',
     $                     IASEED, IXSEED, IYSEED
*
      ELSE
*
*        If in pvm, must participate setting up virtual machine
*
         IF( NNODES.LT.1 ) CALL BLACS_SETUP( IAM, NNODES )
*
*        Temporarily define blacs grid to include all nodes so
*        all nodes have needed startup information
*
         CALL BLACS_GET( -1, 0, ICONTXT )
         CALL BLACS_GRIDINIT( ICONTXT, 'Row-major', 1, NNODES )
*
         CALL IGEBR2D( ICONTXT, 'All', '1-tree', 1, 1, II, 1, 0, 0 )
         CALL IGEBR2D( ICONTXT, 'All', '1-tree', II, 1, WORK, II, 0, 0 )
*
*        Unpack received information
*
         II = 1
         NMAT = WORK(II)
         II = II + 1
         CALL SCOPY( NMAT, WORK(II), 1, MVAL, 1 )
         II = II + NMAT
         CALL SCOPY( NMAT, WORK(II), 1, NVAL, 1 )
         II = II + NMAT
         NBMAT = WORK(II)
         II = II + 1
         CALL SCOPY( NBMAT, WORK(II), 1, MBVAL, 1 )
         II = II + NBMAT
         CALL SCOPY( NBMAT, WORK(II), 1, NBVAL, 1 )
         II = II + NBMAT
         CALL SCOPY( NBMAT, WORK(II), 1, MZVAL, 1 )
         II = II + NBMAT
         CALL SCOPY( NBMAT, WORK(II), 1, NZVAL, 1 )
         II = II + NBMAT
         NGRIDS = WORK(II)
         II = II + 1
         CALL SCOPY( NGRIDS, WORK(II), 1, NPVAL, 1 )
         II = II + NGRIDS
         CALL SCOPY( NGRIDS, WORK(II), 1, NQVAL, 1 )
         II = II + NGRIDS
         XCOMM = CHAR( WORK(II) )
         II = II + 1
         YCOMM = CHAR( WORK(II) )
         II = II + 1
         XWORK = CHAR( WORK(II) )
         II = II + 1
         YWORK = CHAR( WORK(II) )
         II = II + 1
         DIAG =  CHAR( WORK(II) )
         II = II + 1
         BR1ST = CHAR( WORK(II) )
         II = II + 1
         MULLEN = WORK(II)
         II = II + 1
         IAROW = WORK(II)
         II = II + 1
         IACOL = WORK(II)
         II = II + 1
         IXROW = WORK(II)
         II = II + 1
         IXCOL = WORK(II)
         II = II + 1
         IYROW = WORK(II)
         II = II + 1
         IYCOL = WORK(II)
         II = II + 1
         INCX = WORK(II)
         II = II + 1
         INCY = WORK(II)
         II = II + 1
         IASEED = WORK(II)
         II = II + 1
         IXSEED = WORK(II)
         II = II + 1
         IYSEED = WORK(II)
*
         CALL ZGEBR2D( ICONTXT, 'All', '1-tree', 1, 2, TEMP, 1, 0, 0 )
         ALPHA = TEMP( 1 )
         BETA = TEMP( 2 )
      END IF
*
      CALL BLACS_GRIDEXIT( ICONTXT )
*
      RETURN
*
 1000 FORMAT(A)
 2000 FORMAT(2X,A5, 10I6)
 3000 FORMAT(2X,A15,2I6)
 3100 FORMAT(2X,A22,3I6)
 4000 FORMAT(2X,A15,2A6)
 4100 FORMAT(2X,A22,3A4)
 5000 FORMAT(2X,A15,4F8.3)
*
*     End of PBZ2CHKINFO
*
      END
