      SUBROUTINE PDSYEVX( JOBZ, RANGE, UPLO, N, A, IA, JA, DESCA, VL,
     $                    VU, IL, IU, ABSTOL, M, NZ, W, ORFAC, Z, IZ,
     $                    JZ, DESCZ, WORK, LWORK, IWORK, LIWORK, IFAIL,
     $                    ICLUSTR, GAP, INFO )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      CHARACTER          JOBZ, RANGE, UPLO
      INTEGER            IA, IL, INFO, IU, IZ, JA, JZ, LIWORK, LWORK, M,
     $                   N, NZ
      DOUBLE PRECISION   ABSTOL, ORFAC, VL, VU
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( 8 ), DESCZ( 8 ), ICLUSTR( * ),
     $                   IFAIL( * ), IWORK( * )
      DOUBLE PRECISION   A( * ), GAP( * ), W( * ), WORK( * ), Z( * )
*     ..
*
*  Purpose
*  =======
*
*  PDSYEVX computes selected eigenvalues and, optionally, eigenvectors
*  of a real symmetric matrix A by calling the recommended sequence of
*  ScaLAPACK routines.  Eigenvalues/vectors can be selected by
*  specifying a range of values or a range of indices for the desired
*  eigenvalues.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             global context of the operation on the
*                             matrix. The context itself is global.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  PDSYEVX assumes IEEE 754 standard compliant arithmetic.  To port
*  to a system which does not have IEEE 754 arithmetic, modify
*  the appropriate SLmake.inc file to include the compiler switch
*  -DNO_IEEE.  This switch only affects the compilation of pdlaiect.c.
*
*  Arguments
*  =========
*
*     NP = the number of rows local to a given process.
*     NQ = the number of columns local to a given process.
*
*  JOBZ    (global input) CHARACTER*1
*          Specifies whether or not to compute the eigenvectors:
*          = 'N':  Compute eigenvalues only.
*          = 'V':  Compute eigenvalues and eigenvectors.
*
*  RANGE   (global input) CHARACTER*1
*          = 'A': all eigenvalues will be found.
*          = 'V': all eigenvalues in the half-open interval (VL,VU]
*                 will be found.
*          = 'I': the IL-th through IU-th eigenvalues will be found.
*
*  UPLO    (global input) CHARACTER*1
*          Specifies whether the upper or lower triangular part of the
*          symmetric matrix A is stored:
*          = 'U':  Upper triangular
*          = 'L':  Lower triangular
*
*  N       (global input) INTEGER
*          The number of rows and columns of the matrix A.  N >= 0.
*
*  A       (local input/workspace) block cyclic DOUBLE PRECISION array,
*          global dimension (N, N), local dimension (DESCA(8), NQ)
*
*          On entry, the symmetric matrix A.  If UPLO = 'U', only the
*          upper triangular part of A is used to define the elements of
*          the symmetric matrix.  If UPLO = 'L', only the lower
*          triangular part of A is used to define the elements of the
*          symmetric matrix.
*
*          On exit, the lower triangle (if UPLO='L') or the upper
*          triangle (if UPLO='U') of A, including the diagonal, is
*          destroyed.
*
*
*  IA      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix A to operate on.
*
*
*  JA      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix A to operate on.
*
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  VL      (global input) DOUBLE PRECISION
*          If RANGE='V', the lower bound of the interval to be searched
*          for eigenvalues.  Not referenced if RANGE = 'A' or 'I'.
*
*  VU      (global input) DOUBLE PRECISION
*          If RANGE='V', the upper bound of the interval to be searched
*          for eigenvalues.  Not referenced if RANGE = 'A' or 'I'.
*
*  IL      (global input) INTEGER
*          If RANGE='I', the index (from smallest to largest) of the
*          smallest eigenvalue to be returned.  IL >= 1.
*          Not referenced if RANGE = 'A' or 'V'.
*
*  IU      (global input) INTEGER
*          If RANGE='I', the index (from smallest to largest) of the
*          largest eigenvalue to be returned.  min(IL,N) <= IU <= N.
*          Not referenced if RANGE = 'A' or 'V'.
*
*  ABSTOL  (global input) DOUBLE PRECISION
*          If JOBZ='V', setting ABSTOL to PDLAMCH( CONTEXT, 'U') yields
*          the most orthogonal eigenvectors.
*
*          The absolute error tolerance for the eigenvalues.
*          An approximate eigenvalue is accepted as converged
*          when it is determined to lie in an interval [a,b]
*          of width less than or equal to
*
*                  ABSTOL + EPS *   max( |a|,|b| ) ,
*
*          where EPS is the machine precision.  If ABSTOL is less than
*          or equal to zero, then EPS*norm(A) will be used in its place,
*          where norm(A) is the 1-norm of A.  For most problems
*          this is the appropriate level of accuracy to request.
*          For certain strongly graded matrices, greater accuracy
*          can be obtained in very small eigenvalues by setting
*          ABSTOL to some very small positive number.
*          (Note, however, that if ABSTOL is less than sqrt(unfl),
*          where unfl is the underflow threshold, then sqrt(unfl)
*          will be used in its place.)
*
*          See "Computing Small Singular Values of Bidiagonal Matrices
*          with Guaranteed High Relative Accuracy," by Demmel and
*          Kahan, LAPACK Working Note #3.
*
*          See "On the correctness of Parallel Bisection in Floating
*          Point" by Demmel, Dhillon and Ren, LAPACK Working Note #70
*
*  M       (global output) INTEGER
*          Total number of eigenvalues found.  0 <= M <= N.
*
*  NZ      (global output) INTEGER
*          Total number of eigenvectors computed.  0 <= NZ <= M.
*          The number of columns of Z that are filled.
*          If JOBZ .NE. 'V', NZ is not referenced.
*          If JOBZ .EQ. 'V', NZ = M unless the user supplies
*          insufficient space and PDSYEVX is not able to detect this
*          before beginning computation.  To get all the eigenvectors
*          requested, the user must supply both sufficient
*          space to hold the eigenvectors in Z (M .LE. DESCZ(2))
*          and sufficient workspace to compute them.  (See LWORK below.)
*          PDSYEVX is always able to detect insufficient space without
*          computation unless RANGE .EQ. 'V'.
*
*  W       (global output) DOUBLE PRECISION array, dimension (N)
*          On normal exit, the first M entries contain the selected
*          eigenvalues in ascending order.
*
*  ORFAC   (global input) DOUBLE PRECISION
*          Specifies which eigenvectors should be reorthogonalized.
*          Eigenvectors that correspond to eigenvalues which are within
*          tol=ORFAC*norm(A) of each other are to be reorthogonalized.
*          However, if the workspace is insufficient (see LWORK),
*          tol may be decreased until all eigenvectors to be
*          reorthogonalized can be stored in one process.
*          No reorthogonalization will be done if ORFAC equals zero.
*          A default value of 10^-3 is used if ORFAC is negative.
*          ORFAC should be identical on all processes.
*
*  Z       (local output) DOUBLE PRECISION array,
*          global dimension (N, N), local dimension (DESCZ(7), NQ)
*          If JOBZ = 'V', then on normal exit the first M columns of Z
*          contain the orthonormal eigenvectors of the matrix
*          corresponding to the selected eigenvalues.  If an eigenvector
*          fails to converge, then that column of Z contains the latest
*          approximation to the eigenvector, and the index of the
*          eigenvector is returned in IFAIL.
*          If JOBZ = 'N', then Z is not referenced.
*
*
*  IZ      (global input) INTEGER
*          The global row index of the submatrix of the distributed
*          matrix Z to operate on.
*
*
*  JZ      (global input) INTEGER
*          The global column index of the submatrix of the distributed
*          matrix Z to operate on.
*
*
*  DESCZ   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix Z.
*
*  WORK    (local workspace/output) DOUBLE PRECISION array,
*             dimension (LWORK)
*          Version 1.0:  on output, WORK(1) returns the workspace
*          needed to guarantee completion, but not orthogonality
*          of the eigenvectors.
*          If the input parameters are incorrect, WORK(1) may also be
*          incorrect.
*
*          Later we will modify this so if enough workspace is given
*          to complete the request, WORK(1) will return the amount
*          of workspace needed to guarantee orthogonality.  This is
*          described in great detail below:
*          if INFO>=0, then
*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
*             if JOBZ='V' WORK(1) = minimal workspace required to
*               guarantee orthogonal eigenvectors on the given input
*               matrix with the given ORTOL.  (In version 1.0 WORK(1) =
*               minimal workspace required to compute eigenvalues.)
*          if INFO <0 then
*             if JOBZ='N' WORK(1) = minimal=optimal amount of workspace
*             if JOBZ='V' then
*                if RANGE='A' or RANGE='I' then
*                   WORK(1) = minimal workspace required to compute
*                   all eigenvectors (no guarantee on orthogonality)
*                if RANGE='V' then
*                   WORK(1) = minimal workspace required to compute
*                   N_Z = DESCZ(2) eigenvectors (no guarantee on
*                   orthogonality.)  (In version 1.0 WORK(1) =
*                   minimal workspace required to compute eigenvalues.)
*
*  LWORK   (local input) INTEGER
*          See below for definitions of variables used to define LWORK.
*          If no eigenvectors are requested (JOBZ = 'N') then
*             LWORK >= 5*N + MAX( 5*NN, NB*(NP+1) )
*          If eigenvectors are requested (JOBZ = 'V' ) then
*             the amount of workspace required to guarantee that all
*             eigenvectors are computed is:
*                WORK >= 5*N + MAX( 5*NN, NP0 * NQ0 )
*                             + ICEIL( NEIG, NPROW*NPCOL)*NN + 2*NB*NB
*             The computed eigenvectors may not be orthogonal if the
*             minimal workspace is supplied and ORTOL is too small.
*             If you want to guarantee orthogonality (at the cost
*             of potentially poor performance) you should add
*             the following to LWORK:
*                (CLUSTERSIZE-1)*N
*             where CLUSTERSIZE is the number of eigenvalues in the
*             largest cluster, where a cluster is defined as a set of
*             close eigenvalues: { W(K),...,W(K+CLUSTERSIZE-1) |
*                                  W(J+1) <= W(J) + ORFAC*norm(A) }
*          Variable definitions:
*             NN = MAX( N, NB, 2 )
*             NEIG = number of eigenvectors requested
*             NB = DESCA( 3 ) = DESCA( 4 ) = DESCZ( 3 ) = DESCZ( 4 )
*             DESCA( 5 ) = DESCA( 4 ) = DESCZ( 5 ) = DESCZ( 6 ) = 0
*             IA = JA = IZ = JZ = 1
*             NP = NUMROC( N, NB, MYROW, 0, NPROW )
*             NP0 = NUMROC( NN, NB, 0, 0, NPROW )
*             NQ0 = MAX( NUMROC( NEIG, NB, 0, 0, NPCOL ), NB )
*             ICEIL( X, Y ) is a ScaLAPACK function returning
*             ceiling(X/Y)
*
*          When LWORK is too small:
*             If LWORK is too small to guarantee orthogonality,
*             PDSYEVX attempts to maintain orthogonality in
*             the clusters with the smallest
*             spacing between the eigenvalues.
*             If LWORK is too small to compute all the eigenvectors
*             requested, no computation is performed and INFO=-23
*             is returned.  Note that when RANGE='V', PDSYEVX does
*             not know how many eigenvectors are requested until
*             the eigenvalues are computed.  Therefore, when RANGE='V'
*             and as long as LWORK is large enough to allow PDSYEVX to
*             compute the eigenvalues, PDSYEVX will compute the
*             eigenvalues and as many eigenvectors as it can.
*
*          Relationship between workspace, orthogonality & performance:
*             If CLUSTERSIZE >= N/SQRT(NPROW*NPCOL), then providing
*             enough space to compute all the eigenvectors
*             orthogonally will cause serious degradation in
*             performance. In the limit (i.e. CLUSTERSIZE = N-1)
*             PDSTEIN will perform no better than DSTEIN on 1 processor.
*             For CLUSTERSIZE = N/SQRT(NPROW*NPCOL) reorthogonalizing
*             all eigenvectors will increase the total execution time
*             by a factor of 2 or more.
*             For CLUSTERSIZE > N/SQRT(NPROW*NPCOL) execution time will
*             grow as the square of the cluster size, all other factors
*             remaining equal and assuming enough workspace.  Less
*             workspace means less reorthogonalization but faster
*             execution.
*
*  IWORK   (local workspace) INTEGER array
*          On return, IWORK(1) contains the amount of integer workspace
*          required.
*          If the input parameters are incorrect, IWORK(1) may also be
*          incorrect.
*
*  LIWORK  (local input) INTEGER
*          size of IWORK
*          LIWORK >= MAX( ISIZESTEIN, ISIZESTEBZ) + 2*N
*          Where:
*             ISIZESTEIN = 3*N + NPROCS + 1
*             ISIZESTEBZ = MAX( 4*N, 14 )
*
*  IFAIL   (global output) INTEGER array, dimension (N)
*          If JOBZ = 'V', then on normal exit, the first M elements of
*          IFAIL are zero.  If INFO > 0 on exit, then IFAIL contains the
*          indices of the eigenvectors that failed to converge.
*          If JOBZ = 'N', then IFAIL is not referenced.
*
*  ICLUSTR (global output) integer array, dimension (2*NPROW*NPCOL)
*          This array contains indices of eigenvectors corresponding to
*          a cluster of eigenvalues that could not be reorthogonalized
*          due to insufficient workspace (see LWORK, ORFAC and INFO).
*          Eigenvectors corresponding to clusters of eigenvalues indexed
*          ICLUSTR(2*I-1) to ICLUSTR(2*I), could not be
*          reorthogonalized due to lack of workspace. Hence the
*          eigenvectors corresponding to these clusters may not be
*          orthogonal.  ICLUSTR() is a zero terminated array.
*          (ICLUSTR(2*K).NE.0 .AND. ICLUSTR(2*K+1).EQ.0) if and only if
*          K is the number of clusters
*          ICLUSTR is not referenced if JOBZ = 'N'
*
*  GAP     (global output) DOUBLE PRECISION array,
*             dimension (NPROW*NPCOL)
*          This array contains the gap between eigenvalues whose
*          eigenvectors could not be reorthogonalized. The output
*          values in this array correspond to the clusters indicated
*          by the array ICLUSTR. As a result, the dot product between
*          eigenvectors correspoding to the I^th cluster may be as high
*          as ( C * n ) / GAP(I) where C is a small constant.
*
*  INFO    (global output) INTEGER
*
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*          > 0:  if (MOD(INFO,1).NE.0), then one or more eigenvectors
*                  failed to converge.  Their indices are stored
*                  in IFAIL.  send e-mail to scalapack@cs.utk.edu
*                if (MOD(INFO/2,1).NE.0),then eigenvectors corresponding
*                  to one or more clusters of eigenvalues could not be
*                  reorthogonalized because of insufficient workspace.
*                  The indices of the clusters are stored in the array
*                  ICLUSTR.
*                if (MOD(INFO/4,1).NE.0), then space limit prevented
*                  PDSYEVX from computing all of the eigenvectors
*                  between VL and VU.  The number of eigenvectors
*                  computed is returned in NZ.
*                if (MOD(INFO/8,1).NE.0), then PDSTEBZ failed to compute
*                  eigenvalues.
*                  Send e-mail to scalapack@cs.utk.edu
*
*  =====================================================================
*
*  Differences between PDSYEVX and DSYEVX
*  ======================================
*
*  A, LDA -> A, IA, JA, DESCA
*  Z, LDZ -> Z, IZ, JZ, DESCZ
*  WORKSPACE needs are larger for PDSYEVX.
*  LIWORK parameter added
*
*  ORFAC, ICLUSTER() and GAP() parameters added
*  meaning of INFO is changed
*
*  Functional differences:
*    PDSYEVX does not promise orthogonality for eigenvectors associated
*  with tighly clustered eigenvalues.
*  PDSYEVX does not reorthogonalize eigenvectors
*  that are on different processes. The extent of reorthogonalization
*  is controlled by the input parameter LWORK.
*
*  Version 1.0 limitations:
*     DESCA(3) = DESCA(4)
*     IA = JA = 1
*     IZ = JZ = 1
*     DESCA(5) = DESCA(6) = 0
*     DESCZ(5) = DESCZ(6) = 0
*     DESCA(1) = DESCZ(1)
*     DESCA(3) = DESCZ(3)
*     DESCA(4) = DESCZ(4)
*     DESCA(5) = DESCZ(5)
*     DESCA(6) = DESCZ(6)
*     DESCA(7) = DESCZ(7)
*
*     .. Parameters ..
      DOUBLE PRECISION   ZERO, ONE
      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0 )
      INTEGER            IERREIN, IERRCLS, IERRSPC, IERREBZ
      PARAMETER          ( IERREIN = 1, IERRCLS = 2, IERRSPC = 4,
     $                   IERREBZ = 8 )
*     ..
*     .. Local Scalars ..
      LOGICAL            ALLEIG, INDEIG, LOWER, QUICKRETURN, VALEIG,
     $                   WANTZ
      CHARACTER          ORDER
      INTEGER            IINFO, INDD, INDD2, INDE, INDE2, INDIBL,
     $                   INDISP, INDTAU, INDWORK, ISCALE, ISIZESTEBZ,
     $                   ISIZESTEIN, LALLWORK, LIWMIN, LLWORK, LWMIN,
     $                   MAXEIGS, MP_Z, MYCOL, MYROW, NB, NEIG, NN, NP,
     $                   NP0, NPCOL, NPROCS, NPROW, NQ0, NSPLIT, OFFSET,
     $                   SIZEORMTR, SIZESTEIN, SIZESYEVX
      DOUBLE PRECISION   ABSTLL, ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN,
     $                   SIGMA, SMLNUM, VLL, VUU
*     ..
*     .. Local Arrays ..
      INTEGER            IDUM1( 1 ), IDUM2( 1 )
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, NUMROC
      DOUBLE PRECISION   PDLAMCH, PDLANSY
      EXTERNAL           LSAME, ICEIL, NUMROC, PDLAMCH, PDLANSY
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, CHK1MAT, DSCAL, IGAMN2D,
     $                   PCHK1MAT, PDLARED1D, PDLASCL, PDORMTR, PDSTEBZ,
     $                   PDSTEIN, PDSYTRD, PXERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, DBLE, ICHAR, MAX, MIN, MOD, SQRT
*     ..
*     .. Executable Statements ..
*
      QUICKRETURN = ( N.EQ.0 )
*
*     Test the input parameters.
*
      CALL BLACS_GRIDINFO( DESCA( 7 ), NPROW, NPCOL, MYROW, MYCOL )
      INFO = 0
      LIWMIN = -1
      LWMIN = -1
*
      IF( NPROW.EQ.-1 ) THEN
         INFO = -607
      ELSE
         CALL CHK1MAT( N, 4, N, 4, IA, JA, DESCA, 8, INFO )
         CALL CHK1MAT( N, 4, N, 4, IZ, JZ, DESCZ, 21, INFO )
*
         IF( INFO.EQ.0 ) THEN
            NPROCS = NPROW*NPCOL
            LOWER = LSAME( UPLO, 'L' )
            WANTZ = LSAME( JOBZ, 'V' )
            ALLEIG = LSAME( RANGE, 'A' )
            VALEIG = LSAME( RANGE, 'V' )
            INDEIG = LSAME( RANGE, 'I' )
*
*     Set up pointers into the WORK array
*
            INDTAU = 2
            INDE = INDTAU + N
            INDD = INDE + N
            INDD2 = INDD + N
            INDE2 = INDD2 + N
            INDWORK = INDE2 + N
            LLWORK = LWORK - INDWORK + 1
*
*     Set up pointers into the IWORK array
*
            ISIZESTEIN = 3*N + NPROCS + 1
            ISIZESTEBZ = MAX( 4*N, 14 )
            INDIBL = ( MAX( ISIZESTEIN, ISIZESTEBZ ) ) + 1
            INDISP = INDIBL + N
            LIWMIN = MAX( ISIZESTEIN, ISIZESTEBZ ) + 2*N
*
*
            INFO = 0
            M = 0
            IF( LSAME( JOBZ, 'V' ) )
     $         NZ = 0
*
*     Compute the total amount of space needed
*
            NB = DESCA( 3 )
            NN = MAX( N, NB, 2 )
            NP = NUMROC( N, NB, MYROW, 0, NPROW )
            NP0 = NUMROC( NN, NB, 0, 0, NPROW )
            IF( LSAME( JOBZ, 'N' ) .OR. LSAME( RANGE, 'V' ) ) THEN
               LWMIN = 5*N + MAX( 5*NN, NB*( NP+1 ) )
               NEIG = 0
            ELSE
               IF( LSAME( RANGE, 'A' ) ) THEN
                  NEIG = N
               ELSE IF( LSAME( RANGE, 'I' ) ) THEN
                  NEIG = IU - IL + 1
               END IF
               NQ0 = NUMROC( MAX( NEIG, NB ), NB, 0, 0, NPCOL )
               LWMIN = 5*N + 2*NB*NB + ICEIL( NEIG, NPROW*NPCOL )*NN +
     $                 MAX( 5*NN, NP0*NQ0 )
*
            END IF
*
         END IF
         IF( INFO.EQ.0 ) THEN
            IF( .NOT.( WANTZ .OR. LSAME( JOBZ, 'N' ) ) ) THEN
               INFO = -1
            ELSE IF( .NOT.( ALLEIG .OR. VALEIG .OR. INDEIG ) ) THEN
               INFO = -2
            ELSE IF( .NOT.( LOWER .OR. LSAME( UPLO, 'U' ) ) ) THEN
               INFO = -3
            ELSE IF( VALEIG .AND. N.GT.0 .AND. VU.LE.VL ) THEN
               INFO = -10
            ELSE IF( INDEIG .AND. IL.LT.1 ) THEN
               INFO = -11
            ELSE IF( INDEIG .AND. ( IU.LT.MIN( N, IL ) .OR. IU.GT.N ) )
     $                THEN
               INFO = -12
            ELSE IF( LWORK.LT.LWMIN ) THEN
               INFO = -23
            ELSE IF( LIWORK.LT.LIWMIN ) THEN
               INFO = -25
*
*       Version 1.0 Limitations
*
            ELSE IF( IA.NE.1 ) THEN
               INFO = -6
            ELSE IF( JA.NE.1 ) THEN
               INFO = -7
            ELSE IF( IZ.NE.1 ) THEN
               INFO = -19
            ELSE IF( JZ.NE.1 ) THEN
               INFO = -20
            ELSE IF( DESCA( 3 ).NE.DESCA( 4 ) ) THEN
               INFO = -804
            ELSE IF( DESCA( 5 ).NE.0 ) THEN
               INFO = -805
            ELSE IF( DESCA( 6 ).NE.0 ) THEN
               INFO = -806
            ELSE IF( DESCA( 1 ).NE.DESCZ( 1 ) ) THEN
               INFO = -2101
            ELSE IF( DESCA( 3 ).NE.DESCZ( 3 ) ) THEN
               INFO = -2103
            ELSE IF( DESCA( 4 ).NE.DESCZ( 4 ) ) THEN
               INFO = -2104
            ELSE IF( DESCA( 5 ).NE.DESCZ( 5 ) ) THEN
               INFO = -2105
            ELSE IF( DESCA( 6 ).NE.DESCZ( 6 ) ) THEN
               INFO = -2106
            ELSE IF( DESCA( 7 ).NE.DESCZ( 7 ) ) THEN
               INFO = -2107
            END IF
         END IF
         IF( LOWER ) THEN
            IDUM1( 1 ) = ICHAR( 'L' )
         ELSE
            IDUM1( 1 ) = ICHAR( 'U' )
         END IF
         IDUM2( 1 ) = 3
         CALL PCHK1MAT( N, 4, N, 4, IA, JA, DESCA, 8, 1, IDUM1, IDUM2,
     $                  INFO )
         CALL PCHK1MAT( N, 4, N, 4, IA, JA, DESCA, 21, 0, IDUM1, IDUM2,
     $                  INFO )
      END IF
*
      IF( INFO.NE.0 ) THEN
*
         WORK( 1 ) = DBLE( LWMIN )
         IWORK( 1 ) = LIWMIN
         CALL PXERBLA( DESCA( 7 ), 'PDSYEVX', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( QUICKRETURN ) THEN
         IF( LSAME( JOBZ, 'V' ) ) THEN
            NZ = 0
            ICLUSTR( 1 ) = 0
         END IF
         M = 0
         WORK( 1 ) = DBLE( LWMIN )
         IWORK( 1 ) = LIWMIN
         RETURN
      END IF
*
*     Get machine constants.
*
      SAFMIN = PDLAMCH( DESCA( 7 ), 'Safe minimum' )
      EPS = PDLAMCH( DESCA( 7 ), 'Precision' )
      SMLNUM = SAFMIN / EPS
      BIGNUM = ONE / SMLNUM
      RMIN = SQRT( SMLNUM )
      RMAX = MIN( SQRT( BIGNUM ), ONE / SQRT( SQRT( SAFMIN ) ) )
*
*     Scale matrix to allowable range, if necessary.
*
      ABSTLL = ABSTOL
      ISCALE = 0
      IF( VALEIG ) THEN
         VLL = VL
         VUU = VU
      ELSE
         VLL = ZERO
         VUU = ZERO
      END IF
*
      ANRM = PDLANSY( '1', UPLO, N, A, IA, JA, DESCA, WORK( INDWORK ) )
*
      IF( ANRM.GT.ZERO .AND. ANRM.LT.RMIN ) THEN
         ISCALE = 1
         SIGMA = RMIN / ANRM
      ELSE IF( ANRM.GT.RMAX ) THEN
         ISCALE = 1
         SIGMA = RMAX / ANRM
      END IF
*
      IF( ISCALE.EQ.1 ) THEN
         CALL PDLASCL( UPLO, ONE, SIGMA, N, N, A, IA, JA, DESCA, IINFO )
         IF( ABSTOL.GT.0 )
     $      ABSTLL = ABSTOL*SIGMA
         IF( VALEIG ) THEN
            VLL = VL*SIGMA
            VUU = VU*SIGMA
            IF( VUU.EQ.VLL ) THEN
               VUU = VUU + 2*MAX( ABS( VUU )*EPS, SAFMIN )
            END IF
         END IF
      END IF
*
*     Call PDSYTRD to reduce symmetric matrix to tridiagonal form.
*
      LALLWORK = LLWORK
*
      CALL PDSYTRD( UPLO, N, A, IA, JA, DESCA, WORK( INDD ),
     $              WORK( INDE ), WORK( INDTAU ), WORK( INDWORK ),
     $              LLWORK, IINFO )
*
*
*     Copy the values of D, E to all processes
*
*
      CALL PDLARED1D( N, IA, JA, DESCA, WORK( INDD ), WORK( INDD2 ),
     $                WORK( INDWORK ), LLWORK )
*
      CALL PDLARED1D( N, IA, JA, DESCA, WORK( INDE ), WORK( INDE2 ),
     $                WORK( INDWORK ), LLWORK )
*
*     Call PDSTEBZ and, if eigenvectors are desired, PDSTEIN.
*
      IF( WANTZ ) THEN
         ORDER = 'b'
      ELSE
         ORDER = 'e'
      END IF
*
      OFFSET = 0
      IF( LSAME( UPLO, 'U' ) )
     $   OFFSET = 1
*
*
      CALL PDSTEBZ( DESCA( 7 ), RANGE, ORDER, N, VLL, VUU, IL, IU,
     $              ABSTLL, WORK( INDD2 ), WORK( INDE2+OFFSET ), M,
     $              NSPLIT, W, IWORK( INDIBL ), IWORK( INDISP ),
     $              WORK( INDWORK ), LLWORK, IWORK( 1 ), ISIZESTEBZ,
     $              IINFO )
*
      IF( IINFO.NE.0 )
     $   INFO = INFO + IERREBZ
      IF( WANTZ ) THEN
*
         IF( LSAME( RANGE, 'V' ) ) THEN
*
*           Compute the maximum number of eigenvalues that we can
*           compute in the
*           workspace that we have, and that we can store in Z.
*
*           First we compute an approximate value, then we just
*           loop through the possibilities looking for the largest
*           NZ that we can feed to PDSTEIN and PDORMTR
*
*           Since all processes must end up with the same value
*           of NZ, we first compute the minimum of LALLWORK
*
            CALL IGAMN2D( DESCA( 7 ), 'A', ' ', 1, 1, LALLWORK, 1, 1, 1,
     $                    -1, -1, -1 )
*
            IF( LALLWORK.LT.10*N ) THEN
               MAXEIGS = ( LALLWORK-5*N ) / N
            ELSE
               MAXEIGS = LALLWORK / ( 2*N )
            END IF
*
            MAXEIGS = MAXEIGS*NPROW*NPCOL
            MAXEIGS = MIN( MAXEIGS, DESCZ( 2 ) )
*
            MP_Z = NUMROC( N, NB, MYROW, DESCZ( 5 ), NPROW )
*
            DO 10 NZ = MIN( MAXEIGS, M ), 0, -1
               NQ0 = NUMROC( NZ, NB, 0, 0, NPCOL )
               SIZESTEIN = ICEIL( NZ, NPROCS )*N + MAX( 5*N, NP0*NQ0 )
               SIZEORMTR = MAX( ( NB*( NB-1 ) ) / 2, ( NQ0+MP_Z )*NB ) +
     $                     NB*NB
*
               SIZESYEVX = 5*N + MAX( SIZESTEIN, SIZEORMTR )
               IF( SIZESYEVX.LT.LWORK )
     $            GO TO 20
   10       CONTINUE
   20       CONTINUE
         ELSE
            NZ = M
         END IF
         IF( NZ.NE.M )
     $      INFO = INFO + IERRSPC
*
         CALL PDSTEIN( N, WORK( INDD2 ), WORK( INDE2+OFFSET ), NZ, W,
     $                 IWORK( INDIBL ), IWORK( INDISP ), ORFAC, Z, IZ,
     $                 JZ, DESCZ, WORK( INDWORK ), LALLWORK, IWORK( 1 ),
     $                 ISIZESTEIN, IFAIL, ICLUSTR, GAP, IINFO )
*
         IF( IINFO.GE.M+1 )
     $      INFO = INFO + IERRCLS
         IF( MOD( IINFO, M+1 ).NE.0 )
     $      INFO = INFO + IERREIN
*
         IF( LSAME( UPLO, 'U' ) ) THEN
*
*     Upper triangular storage
*
*     Z = Q * Z
*
*
            CALL PDORMTR( 'L', UPLO, 'N', N, NZ, A, IA, JA, DESCA,
     $                    WORK( INDTAU ), Z, IZ, JZ, DESCZ,
     $                    WORK( INDWORK ), LLWORK, IINFO )
*
         ELSE
*
*     Lower triangular storage
*
*     Z = Q * Z
*
*
            CALL PDORMTR( 'L', UPLO, 'N', N, NZ, A, IA, JA, DESCA,
     $                    WORK( INDTAU ), Z, IZ, JZ, DESCZ,
     $                    WORK( INDWORK ), LLWORK, IINFO )
*
*       end of lower triangular storage
*
         END IF
*
*       endif (WANTZ)
*
      END IF
*
*     If matrix was scaled, then rescale eigenvalues appropriately.
*
      IF( ISCALE.EQ.1 ) THEN
         CALL DSCAL( M, ONE / SIGMA, W, 1 )
      END IF
*
*
*
      WORK( 1 ) = DBLE( LWMIN )
      IWORK( 1 ) = LIWMIN
*
      RETURN
*
*     End of PDSYEVX
*
      END
