      SUBROUTINE PZGEBD2( M, N, A, IA, JA, DESCA, D, E, TAUQ, TAUP,
     $                    WORK, LWORK, INFO )
*
*  -- ScaLAPACK auxilliary routine (version 1.1) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     March 20, 1995
*
*     .. Scalar Arguments ..
      INTEGER            IA, INFO, JA, LWORK, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * )
      COMPLEX*16          A( * ), D( * ), E( * ), TAUP( * ), TAUQ( * ),
     $                   WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PZGEBD2 reduces a complex general M-by-N distributed matrix
*  sub( A ) = A(IA:IA+M-1,JA:JA+N-1) to upper or lower bidiagonal
*  form B by an unitary transformation: Q' * sub( A ) * P = B.
*
*  If M >= N, B is upper bidiagonal; if M < N, B is lower bidiagonal.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the handle
*                             (the integer value) may vary.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  M       (global input) INTEGER
*          The number of rows to be operated on, i.e. the number of rows
*          of the distributed submatrix sub( A ). M >= 0.
*
*  N       (global input) INTEGER
*          The number of columns to be operated on, i.e. the number of
*          columns of the distributed submatrix sub( A ). N >= 0.
*
*  A       (local input/local output) COMPLEX*16 pointer into the
*          local memory to an array of dimension (LLD_A,LOCq(JA+N-1)).
*          On entry, this array contains the local pieces of the
*          general distributed matrix sub( A ). On exit, if M >= N,
*          the diagonal and the first superdiagonal of sub( A ) are
*          overwritten with the upper bidiagonal matrix B; the elements
*          below the diagonal, with the array TAUQ, represent the
*          unitary matrix Q as a product of elementary reflectors, and
*          the elements above the first superdiagonal, with the array
*          TAUP, represent the orthogonal matrix P as a product of
*          elementary reflectors. If M < N, the diagonal and the first
*          subdiagonal are overwritten with the lower bidiagonal
*          matrix B; the elements below the first subdiagonal, with the
*          array TAUQ, represent the unitary matrix Q as a product of
*          elementary reflectors, and the elements above the diagonal,
*          with the array TAUP, represent the orthogonal matrix P as a
*          product of elementary reflectors. See Further Details.
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  D       (local output) COMPLEX*16 array, dimension
*          LOCq(JA+MIN(M,N)-1) if M >= N; LOCp(IA+MIN(M,N)-1) otherwise.
*          The distributed diagonal elements of the bidiagonal matrix
*          B: D(i) = A(i,i). D is tied to the distributed matrix A.
*
*  E       (local output) COMPLEX*16 array, dimension
*          LOCp(IA+MIN(M,N)-1) if M >= N; LOCq(JA+MIN(M,N)-2) otherwise.
*          The distributed off-diagonal elements of the bidiagonal
*          distributed matrix B:
*          if m >= n, E(i) = A(i,i+1) for i = 1,2,...,n-1;
*          if m < n, E(i) = A(i+1,i) for i = 1,2,...,m-1.
*          E is tied to the distributed matrix A.
*
*  TAUQ    (local output) COMPLEX*16 array dimension
*          LOCq(JA+MIN(M,N)-1). The scalar factors of the elementary
*          reflectors which represent the unitary matrix Q. TAUQ is
*          tied to the distributed matrix A. See Further Details.
*
*  TAUP    (local output) COMPLEX*16 array, dimension
*          LOCp(IA+MIN(M,N)-1). The scalar factors of the elementary
*          reflectors which represent the unitary matrix P. TAUP is
*          tied to the distributed matrix A. See Further Details.
*
*  WORK    (local workspace) COMPLEX*16 array, dimension (LWORK)
*          On exit, WORK(1) returns the minimal LWORK.
*
*  LWORK   (local input) INTEGER
*          The dimension of the array WORK.
*          LWORK >= MAX( MpA0, NqA0 )
*
*          where NB = MB_A = NB_A, IROFFA = MOD( IA-1, NB )
*          IAROW = INDXG2P( IA, NB, MYROW, RSRC_A, NPROW ),
*          IACOL = INDXG2P( JA, NB, MYCOL, CSRC_A, NPCOL ),
*          MpA0 = NUMROC( M+IROFFA, NB, MYROW, IAROW, NPROW ),
*          NqA0 = NUMROC( N+IROFFA, NB, MYCOL, IACOL, NPCOL ).
*
*          INDXG2P and NUMROC are ScaLAPACK tool functions;
*          MYROW, MYCOL, NPROW and NPCOL can be determined by calling
*          the subroutine BLACS_GRIDINFO.
*
*  INFO    (local output) INTEGER
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*
*  =====================================================================
*
*  Alignment requirements
*  ======================
*
*  The distributed submatrix sub( A ) must verify some alignment proper-
*  ties, namely the following expressions should be true:
*                  ( MB_A.EQ.NB_A .AND. IROFFA.EQ.ICOFFA )
*
*  Further Details
*  ===============
*
*  The matrices Q and P are represented as products of elementary
*  reflectors:
*
*  If m >= n,
*
*     Q = H(1) H(2) . . . H(n)  and  P = G(1) G(2) . . . G(n-1)
*
*  Each H(i) and G(i) has the form:
*
*     H(i) = I - tauq * v * v'  and G(i) = I - taup * u * u'
*
*  where tauq and taup are complex scalars, and v and u are complex
*  vectors;
*  v(1:i-1) = 0, v(i) = 1, and v(i+1:m) is stored on exit in
*  A(ia+i:ia+m-1,ja+i-1);
*  u(1:i) = 0, u(i+1) = 1, and u(i+2:n) is stored on exit in
*  A(ia+i-1,ja+i+1:ja+n-1);
*  tauq is stored in TAUQ(ja+i-1) and taup in TAUP(ia+i-1).
*
*  If m < n,
*
*     Q = H(1) H(2) . . . H(m-1)  and  P = G(1) G(2) . . . G(m)
*
*  Each H(i) and G(i) has the form:
*
*     H(i) = I - tauq * v * v'  and G(i) = I - taup * u * u'
*
*  where tauq and taup are complex scalars, and v and u are complex
*  vectors;
*  v(1:i) = 0, v(i+1) = 1, and v(i+2:m) is stored on exit in
*  A(ia+i+1:ia+m-1,ja+i-1);
*  u(1:i-1) = 0, u(i) = 1, and u(i+1:n) is stored on exit in
*  A(ia+i-1,ja+i:ja+n-1);
*  tauq is stored in TAUQ(ja+i-1) and taup in TAUP(ia+i-1).
*
*  The contents of sub( A ) on exit are illustrated by the following
*  examples:
*
*  m = 6 and n = 5 (m > n):          m = 5 and n = 6 (m < n):
*
*    (  d   e   u1  u1  u1 )           (  d   u1  u1  u1  u1  u1 )
*    (  v1  d   e   u2  u2 )           (  e   d   u2  u2  u2  u2 )
*    (  v1  v2  d   e   u3 )           (  v1  e   d   u3  u3  u3 )
*    (  v1  v2  v3  d   e  )           (  v1  v2  e   d   u4  u4 )
*    (  v1  v2  v3  v4  d  )           (  v1  v2  v3  e   d   u5 )
*    (  v1  v2  v3  v4  v5 )
*
*  where d and e denote diagonal and off-diagonal elements of B, vi
*  denotes an element of the vector defining H(i), and ui an element of
*  the vector defining G(i).
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16         ONE, ZERO
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ),
     $                   ZERO = ( 0.0D+0, 0.0D+0 ) )
*     ..
*     .. Local Scalars ..
      INTEGER            I, IACOL, IAROW, ICOFFA, ICTXT, II, IJ, IROFFA,
     $                   J, JJ, K, LWMIN, MP, MPA0, MYCOL, MYROW, NPCOL,
     $                   NPROW, NQ, NQA0
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_ABORT, BLACS_GRIDINFO, CHK1MAT, INFOG2L,
     $                   PXERBLA, PZELSET, PZLACGV, PZLARF,
     $                   PZLARFC, PZLARFG, ZGEBR2D, ZGEBS2D
*     ..
*     .. External Functions ..
      INTEGER            INDXG2P, NUMROC
      EXTERNAL           INDXG2P, NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          DBLE, MAX, MIN, MOD
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
      ICTXT = DESCA( 7 )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Test the input parameters
*
      INFO = 0
      IF( NPROW.EQ.-1 ) THEN
         INFO = -607
      ELSE
         CALL CHK1MAT( M, 1, N, 2, IA, JA, DESCA, 6, INFO )
         IF( INFO.EQ.0 ) THEN
            IROFFA = MOD( IA-1, DESCA( 3 ) )
            ICOFFA = MOD( JA-1, DESCA( 4 ) )
            IAROW = INDXG2P( IA, DESCA( 3 ), MYROW, DESCA( 5 ), NPROW )
            IACOL = INDXG2P( JA, DESCA( 4 ), MYCOL, DESCA( 6 ), NPCOL )
            MPA0 = NUMROC( M+IROFFA, DESCA( 3 ), MYROW, IAROW, NPROW )
            NQA0 = NUMROC( N+ICOFFA, DESCA( 4 ), MYCOL, IACOL, NPCOL )
            LWMIN = MAX( MPA0, NQA0 )
*
            WORK( 1 ) = DBLE( LWMIN )
            IF( IROFFA.NE.ICOFFA ) THEN
               INFO = -5
            ELSE IF( DESCA( 3 ).NE.DESCA( 4 ) ) THEN
               INFO = -604
            ELSE IF( LWORK.LT.LWMIN ) THEN
               INFO = -12
            END IF
         END IF
      END IF
*
      IF( INFO.LT.0 ) THEN
         CALL PXERBLA( ICTXT, 'PZGEBD2', -INFO )
         CALL BLACS_ABORT( ICTXT, 1 )
         RETURN
      END IF
*
      CALL INFOG2L( IA, JA, DESCA, NPROW, NPCOL, MYROW, MYCOL, II, JJ,
     $              IAROW, IACOL )
*
      IF( M.EQ.1 .AND. N.EQ.1 ) THEN
         IF( MYCOL.EQ.IACOL ) THEN
            IF( MYROW.EQ.IAROW ) THEN
               D( JJ ) = A( II+(JJ-1)*DESCA( 8 ) )
               CALL ZGEBS2D( ICTXT, 'Columnwise', ' ', 1, 1, D( JJ ),
     $                       1 )
            ELSE
               CALL ZGEBR2D( ICTXT, 'Columnwise', ' ', 1, 1, D( JJ ),
     $                       1, IAROW, IACOL )
            END IF
            TAUQ( JJ ) = ZERO
         END IF
         IF( MYROW.EQ.IAROW )
     $      TAUP( II ) = ZERO
         RETURN
      END IF
*
      MP = NUMROC( M+IA-1, DESCA( 3 ), MYROW, DESCA( 5 ), NPROW )
      NQ = NUMROC( N+JA-1, DESCA( 4 ), MYCOL, DESCA( 6 ), NPCOL )
*
      IF( M.GE.N ) THEN
*
*        Reduce to upper bidiagonal form
*
         DO 10 K = 1, N
            I = IA + K - 1
            J = JA + K - 1
*
*           Generate elementary reflector H(j) to annihilate
*           A(ia+i:ia+m-1,j)
*
            IJ = MIN( JJ+K-1, NQ )
            CALL PZLARFG( M-K+1, D( IJ ), I, J, A, MIN( I+1, M+IA-1 ),
     $                    J, DESCA, 1, TAUQ )
            CALL PZELSET( A, I, J, DESCA, ONE )
*
*           Apply H(i) to A(i:ia+m-1,i+1:ja+n-1) from the left
*
            CALL PZLARFC( 'Left', M-K+1, N-K, A, I, J, DESCA, 1, TAUQ,
     $                    A, I, J+1, DESCA, WORK )
            CALL PZELSET( A, I, J, DESCA, D( IJ ) )
            IJ = MIN( II+K-1, MP )
*
            IF( K.LT.N ) THEN
*
*              Generate elementary reflector G(i) to annihilate
*              A(i,ja+j+1:ja+n-1)
*
               CALL PZLACGV( N-K, A, I, J+1, DESCA, DESCA( 1 ) )
               CALL PZLARFG( N-K, E( IJ ), I, J+1, A, I,
     $                       MIN( J+2, JA+N-1 ), DESCA, DESCA( 1 ),
     $                       TAUP )
               CALL PZELSET( A, I, J+1, DESCA, ONE )
*
*              Apply G(i) to A(i+1:ia+m-1,i+1:ja+n-1) from the right
*
               CALL PZLARF( 'Right', M-K, N-K, A, I, J+1, DESCA,
     $                      DESCA( 1 ), TAUP, A, I+1, J+1, DESCA, WORK )
               CALL PZLACGV( N-K, A, I, J+1, DESCA, DESCA( 1 ) )
               CALL PZELSET( A, I, J+1, DESCA, E( IJ ) )
            ELSE
               IF( MYROW.EQ.IAROW )
     $            TAUP( IJ ) = ZERO
            END IF
   10    CONTINUE
*
      ELSE
*
*        Reduce to lower bidiagonal form
*
         DO 20 K = 1, M
            I = IA + K - 1
            J = JA + K - 1
*
*           Generate elementary reflector G(i) to annihilate
*           A(i,ja+j:ja+n-1)
*
            CALL PZLACGV( N-K+1, A, I, J, DESCA, DESCA( 1 ) )
            IJ = MIN( II+K-1, MP )
            CALL PZLARFG( N-K+1, D( IJ ), I, J, A, I,
     $                    MIN( J+1, JA+N-1 ), DESCA, DESCA( 1 ), TAUP )
            CALL PZELSET( A, I, J, DESCA, ONE )
*
*           Apply G(i) to A(i:ia+m-1,j:ja+n-1) from the right
*
            CALL PZLARF( 'Right', M-K, N-K+1, A, I, J, DESCA,
     $                   DESCA( 1 ), TAUP, A, MIN( I+1, IA+M-1 ), J,
     $                   DESCA, WORK )
            CALL PZLACGV( N-K+1, A, I, J, DESCA, DESCA( 1 ) )
            CALL PZELSET( A, I, J, DESCA, D( IJ ) )
            IJ = MIN( JJ+K-1, NQ )
*
            IF( K.LT.M ) THEN
*
*              Generate elementary reflector H(i) to annihilate
*              A(i+2:ia+m-1,j)
*
               CALL PZLARFG( M-K, E( IJ ), I+1, J, A,
     $                       MIN( I+2, IA+M-1 ), J, DESCA, 1, TAUQ )
               CALL PZELSET( A, I+1, J, DESCA, ONE )
*
*              Apply H(i) to A(i+1:ia+m-1,j+1:ja+n-1) from the left
*
               CALL PZLARFC( 'Left', M-K, N-K, A, I+1, J, DESCA, 1,
     $                       TAUQ, A, I+1, J+1, DESCA, WORK )
               CALL PZELSET( A, I+1, J, DESCA, E( IJ ) )
            ELSE
               IF( MYCOL.EQ.IACOL )
     $            TAUQ( IJ ) = ZERO
            END IF
   20    CONTINUE
      END IF
*
      RETURN
*
*     End of PZGEBD2
*
      END
