      SUBROUTINE PDSEPREQ( NIN, RESULT, MEM, MEMSIZE, IMEM, IMEMSIZE,
     $                     NOUT, ISEED, INFO )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      INTEGER            IMEMSIZE, INFO, MEMSIZE, NIN, NOUT, RESULT
*     ..
*     .. Array Arguments ..
      INTEGER            IMEM( IMEMSIZE ), ISEED( 4 )
      DOUBLE PRECISION   MEM( MEMSIZE )
*     ..
*
*  Purpose
*  =======
*
*  PDSEPREQ performs one request from the input file 'SEP.dat'
*  A request is the cross product of the specifications in the
*  input file.  PDSEPREQ may print one line per test, or not, depending
*  on the request specification.
*
*  Arguments
*  =========
*
*  NIN      (local input) INTEGER
*           The unit number for the input file 'SEP.dat'
*
*  RESULT   (global output) INTEGER
*           Number of test requests that failed (either 0 or 1)
*           Only valid on process 0
*
*  MEM      (local input) DOUBLE PRECISION ARRAY, dimension MEMSIZE
*           A huge array encompassing the available double prec memory
*
*  MEMSIZE  (local input)  INTEGER
*           Size of MEM array
*
*  IMEM     (local input) INTEGER ARRAY, dimension IMEMSIZE
*           A large array for integer memory
*
*  IMEMSIZE (local input)  INTEGER
*           Size of IMEM array
*
*  NOUT   (local input) INTEGER
*           The unit number for output file.
*           NOUT = 6, output to screen,
*           NOUT = 0, output to stderr.  Only used on node 0.
*
*  ISEED    (global input/output) INTEGER array, dimension 4
*            Random number generator seed
*
*  INFO     (global output) INTEGER
*           0 = test request ran
*           -1 = end of file
*           -2 = incorrect .dat file
*
*
*     .. Parameters ..
*
      INTEGER            MAXSETSIZE
      PARAMETER          ( MAXSETSIZE = 50 )
*     ..
*     .. Local Scalars ..
      INTEGER            IAM, IMIDPAD, INITCON, IPOSTPAD, IPREPAD, LDA,
     $                   LLIWORK, LLWORK, MATSIZE, MATTYPE, N, NB,
     $                   NEIGTYPES, NMATSIZES, NMATTYPES, NNODES, NP,
     $                   NPCOL, NPCONFIGS, NPROW, NQ, NUPLOS, ORDER,
     $                   PCONFIG, PRINTREQ, PTRA, PTRCOPYA, PTRGAP,
     $                   PTRICLUS, PTRIFAIL, PTRIWRK, PTRW, PTRW2,
     $                   PTRWRK, PTRZ, RES, UPLO
      DOUBLE PRECISION   ABSTOL, THRESH
*     ..
*     .. Local Arrays ..
      CHARACTER          UPLOS( 2 )
      INTEGER            MATSIZES( MAXSETSIZE ), MATTYPES( MAXSETSIZE ),
     $                   NBS( MAXSETSIZE ), NPCOLS( MAXSETSIZE ),
     $                   NPROWS( MAXSETSIZE )
*     ..
*     .. External Functions ..
      INTEGER            NUMROC
      EXTERNAL           NUMROC
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GET, BLACS_GRIDEXIT, BLACS_GRIDINIT,
     $                   BLACS_PINFO, PDSEPINFO, PDSEPTST
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
      CALL BLACS_PINFO( IAM, NNODES )
      CALL BLACS_GET( -1, 0, INITCON )
      CALL BLACS_GRIDINIT( INITCON, 'R', 1, NNODES )
*
      CALL PDSEPINFO( INITCON, IAM, NIN, NOUT, MAXSETSIZE, NMATSIZES,
     $                MATSIZES, NUPLOS, UPLOS, NPCONFIGS, NPROWS,
     $                NPCOLS, NBS, NMATTYPES, MATTYPES, 22, NEIGTYPES,
     $                THRESH, ORDER, ABSTOL, PRINTREQ, INFO )
*
      CALL BLACS_GRIDEXIT( INITCON )
*
      IF( INFO.EQ.0 ) THEN
*
         IF( IAM.EQ.0 .AND. PRINTREQ.EQ.1 ) THEN
            WRITE( NOUT, FMT = 9999 )
         END IF
*
         DO 40 MATSIZE = 1, NMATSIZES
*
            DO 30 PCONFIG = 1, NPCONFIGS
*
               DO 20 MATTYPE = 1, NMATTYPES
*
                  DO 10 UPLO = 1, NUPLOS
*
                     N = MATSIZES( MATSIZE )
                     ORDER = N
*
                     NPROW = NPROWS( PCONFIG )
                     NPCOL = NPCOLS( PCONFIG )
                     NB = NBS( PCONFIG )
*
                     NP = NUMROC( N, NB, 0, 0, NPROW )
                     NQ = NUMROC( N, NB, 0, 0, NPCOL )
                     IPREPAD = 0
                     IMIDPAD = 0
                     IPOSTPAD = 0
                     IPREPAD = MAX( NB, NP )
                     IMIDPAD = NB
                     IPOSTPAD = MAX( NB, NQ )
*
                     LDA = MAX( NP, 1 ) + IMIDPAD
*
                     PTRA = 1
                     PTRZ = PTRA + LDA*NQ + IPREPAD + IPOSTPAD
                     PTRCOPYA = PTRZ + LDA*NQ + IPREPAD + IPOSTPAD
                     PTRW = PTRCOPYA + LDA*NQ + IPREPAD + IPOSTPAD
                     PTRW2 = PTRW + N + IPREPAD + IPOSTPAD
                     PTRGAP = PTRW2 + N + IPREPAD + IPOSTPAD
                     PTRWRK = PTRGAP + NPROW*NPCOL + IPREPAD + IPOSTPAD
                     LLWORK = MEMSIZE - PTRWRK + 1 - IPREPAD - IPOSTPAD
*
                     PTRIFAIL = 1
                     PTRICLUS = PTRIFAIL + N + IPREPAD + IPOSTPAD
                     PTRIWRK = PTRICLUS + 2*NPROW*NPCOL + IPREPAD +
     $                         IPOSTPAD
                     LLIWORK = IMEMSIZE - PTRIWRK + 1 - IPREPAD -
     $                         IPOSTPAD
*
                     CALL PDSEPTST( UPLOS( UPLO ), N, NPROW, NPCOL, NB,
     $                              MATTYPES( MATTYPE ), NEIGTYPES,
     $                              THRESH, N, ABSTOL, PRINTREQ, ISEED,
     $                              MEM( PTRA ), MEM( PTRCOPYA ),
     $                              MEM( PTRZ ), LDA, MEM( PTRW ),
     $                              MEM( PTRW2 ), IMEM( PTRIFAIL ),
     $                              IMEM( PTRICLUS ), MEM( PTRGAP ),
     $                              IPREPAD, IPOSTPAD, MEM( PTRWRK ),
     $                              LLWORK, IMEM( PTRIWRK ), LLIWORK,
     $                              NOUT, RES )
*
                     IF( RES.NE.0 )
     $                  RESULT = 1
   10             CONTINUE
   20          CONTINUE
   30       CONTINUE
   40    CONTINUE
      END IF
*
*
 9999 FORMAT( ' RESULT       WALL      CPU      N    P    Q   NB TYP ',
     $      'NT    CHK      QTQ' )
      RETURN
      END
