      SUBROUTINE PDSEPTST( UPLO, N, NPROW, NPCOL, NB, MATTYPE,
     $                     NEIGTYPES, THRESH, ORDER, ABSTOL, PRINTREQ,
     $                     ISEED, A, COPYA, Z, LDA, WIN, WNEW, IFAIL,
     $                     ICLUSTR, GAP, IPREPAD, IPOSTPAD, WORK, LWORK,
     $                     IWORK, LIWORK, NOUT, RESULT )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            IPOSTPAD, IPREPAD, LDA, LIWORK, LWORK, MATTYPE,
     $                   N, NB, NEIGTYPES, NOUT, NPCOL, NPROW, ORDER,
     $                   PRINTREQ, RESULT
      DOUBLE PRECISION   ABSTOL, THRESH
*     ..
*     .. Array Arguments ..
      INTEGER            ICLUSTR( * ), IFAIL( * ), ISEED( 4 ),
     $                   IWORK( * )
      DOUBLE PRECISION   A( LDA, * ), COPYA( LDA, * ), GAP( * ),
     $                   WIN( * ), WNEW( * ), WORK( * ), Z( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  PDSEPTST builds a random matrix, runs PDSYEVX() to
*  compute the eigenvalues
*  and eigenvectors and then performs two tests to
*  determine if the result
*  is good enough.  The two tests:  |AQ -QL| / (abstol + ulp * norm(A) )
*  and  |QT * Q - I| / ulp * norm(A)
*
*  The random matrix built depends upon the following parameters:
*     N, NB, ISEED, ORDER
*
*  Return value
*  ============
*  -3       This process is not involved
*  0        Test succeeded (passed |AQ -QL| and |QT * Q - I| tests)
*  1        At least one test failed
*
*  Arguments
*  =========
*
*     NP = the number of rows local to a given process.
*     NQ = the number of columns local to a given process.
*
*  UPLO     (global input) CHARACTER*1
*           Specifies whether the upper or lower triangular part of the
*           symmetric matrix A is stored:
*           = 'U':  Upper triangular
*           = 'L':  Lower triangular
*
*  N        (global input) INTEGER
*           Size of the matrix to be tested.  (global size)
*
*  NPROW    (global input) INTEGER
*           Number of process rows
*
*  NPCOL    (global input) INTEGER
*           Number of process columns
*
*  NB       (global input) INTEGER
*           The block size of the matrix A.  NB >= 1.
*
*  MATTYPE  (global input) INTEGER
*           Matrix type
*  Currently, the list of possible types is:
*
*  (1)  The zero matrix.
*  (2)  The identity matrix.
*
*  (3)  A diagonal matrix with evenly spaced entries
*       1, ..., ULP  and random signs.
*       (ULP = (first number larger than 1) - 1 )
*  (4)  A diagonal matrix with geometrically spaced entries
*       1, ..., ULP  and random signs.
*  (5)  A diagonal matrix with "clustered" entries 1, ULP, ..., ULP
*       and random signs.
*
*  (6)  Same as (4), but multiplied by SQRT( overflow threshold )
*  (7)  Same as (4), but multiplied by SQRT( underflow threshold )
*
*  (8)  A matrix of the form  U' D U, where U is orthogonal and
*       D has evenly spaced entries 1, ..., ULP with random signs
*       on the diagonal.
*
*  (9)  A matrix of the form  U' D U, where U is orthogonal and
*       D has geometrically spaced entries 1, ..., ULP with random
*       signs on the diagonal.
*
*  (10) A matrix of the form  U' D U, where U is orthogonal and
*       D has "clustered" entries 1, ULP,..., ULP with random
*       signs on the diagonal.
*
*  (11) Same as (8), but multiplied by SQRT( overflow threshold )
*  (12) Same as (8), but multiplied by SQRT( underflow threshold )
*
*  (13) Symmetric matrix with random entries chosen from (-1,1).
*  (14) Same as (13), but multiplied by SQRT( overflow threshold )
*  (15) Same as (13), but multiplied by SQRT( underflow threshold )
*  (16) Same as (8), but diagonal elements are all positive.
*  (17) Same as (9), but diagonal elements are all positive.
*  (18) Same as (10), but diagonal elements are all positive.
*  (19) Same as (16), but multiplied by SQRT( overflow threshold )
*  (20) Same as (16), but multiplied by SQRT( underflow threshold )
*  (21) A tridiagonal matrix that is a direct sum of smaller diagonally
*       dominant submatrices. Each unreduced submatrix has geometrically
*       spaced diagonal entries 1, ..., ULP.
*  (22) A matrix of the form  U' D U, where U is orthogonal and
*       D has ceil(lgN) "clusters" at 0,1,2,...,ceil(lgN)-1. The
*       size of the cluster at the value I is 2^I.
*
*  NEIGTYPES (global input) INTEGER
*           Number of eigenvalue requet types (must be either 1 or 3)
*           NEIGTYPES = 1 means test all eigenvalues and eigenvectors
*           NEIGTYPES = 6 means test:
*               all eigenvalues and eigenvectors
*               test eigenvalues only (whole range)
*               test eigenvalues and vectors within a given range
*               test eigenvalues within a given range
*               test all eigenvalues and vectors chosen by index
*               test all eigenvalues chosen by index
*               test all eigenvalues and vectors with minimum workspace
*               test all eigenvalues and vectors with random workspace
*
*  THRESH   (global input) DOUBLE PRECISION
*          A test will count as "failed" if the "error", computed as
*          described below, exceeds THRESH.  Note that the error
*          is scaled to be O(1), so THRESH should be a reasonably
*          small multiple of 1, e.g., 10 or 100.  In particular,
*          it should not depend on the precision (single vs. double)
*          or the size of the matrix.  It must be at least zero.
*
*  ORDER    (global input) INTEGER
*           Number of reflectors used in test matrix creation.
*           If ORDER is large, it will
*           take more time to create the test matrices but they will
*           be closer to random.
*           ORDER .lt. N not implemented
*
*  ABSTOL   (global input) DOUBLE PRECISION
*           The absolute tolerance for the eigenvalues. An
*           eigenvalue is considered to be located if it has
*           been determined to lie in an interval whose width
*           is "abstol" or less. If "abstol" is less than or equal
*           to zero, then ulp*|T| will be used, where |T| is
*           the 1-norm of the matrix. If eigenvectors are
*           desired later by inverse iteration ("PDSTEIN"),
*           "abstol" MUST NOT be bigger than ulp*|T|.
*
*           For the purposes of this test, ABSTOL=0.0 is fine.
*           THis test does not test for high relative accuracy.
*
*  PRINTREQ (global input) INTEGER
*           1 = Full print (i.e. print from this routine)
*           2 = Summary print only (i.e. no printing in this routine)
*
*  ISEED   (global input/output) INTEGER array, dimension (4)
*          On entry, the seed of the random number generator; the array
*          elements must be between 0 and 4095, and ISEED(4) must be
*          odd.
*          On exit, the seed is updated.
*
*  A       (local workspace) DOUBLE PRECISION array, dim (N*N)
*          global dimension (N, N), local dimension (LDA, NQ)
*            A is distributed in a block cyclic manner over both rows
*            and columns.  The actual location of a particular element
*            in A is controlled by the values of NPROW, NPCOL, and NB.
*          The test matrix, which is then modified by PDSYEVX
*
*  COPYA   (local workspace) DOUBLE PRECISION array, dim (N, N)
*          COPYA is used to hold an identical copy of the array A
*          identical in both form and content to A
*
*  Z       (local workspace) DOUBLE PRECISION array, dim (N*N)
*            Z is distributed in the same manner as A
*          Z is used as workspace by the test routines
*          pdsepchk,f and pdsepqtq.f
*
*  W       (local workspace) DOUBLE PRECISION array, dimension (N)
*          On normal exit from PDSYEVX, the first M entries
*          contain the selected eigenvalues in ascending order.
*
*  IFAIL   (global workspace) INTEGER array, dimension (N)
*
*  WORK    (local workspace) DOUBLE PRECISION array, dimension (LWORK)
*
*  LWORK   (local input) INTEGER
*          The length of the array WORK.  LWORK >= max(1,??)
*
*  IWORK   (local workspace) INTEGER array, dimension (LIWORK)
*
*  LIWORK  (local input) INTEGER
*          The length of the array IWORK.  LIWORK >= max(1,??)
*
*  NOUT   (local input) INTEGER
*           The unit number for output file.  Only used on node 0.
*           NOUT = 6, output to screen,
*           NOUT = 0, output to stderr.
*
*  RESULT (global output) INTEGER
*         0 = test passed all residual and concsistency checks
*         <>0 = test failed
*
*
*     .. Parameters ..
      DOUBLE PRECISION   ZERO, ONE, TWO, TEN, HALF
      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0,
     $                   TEN = 10.0D0, HALF = ONE / TWO )
      INTEGER            MAXTYP
      PARAMETER          ( MAXTYP = 22 )
*     ..
*
*     .. Local Scalars ..
      INTEGER            CONTEXT, CSRC_A, I, IACOL, IAM, IAROW, ICOFFA,
     $                   IL, IMODE, IN, INFO, IROFFA, ITYPE, IU, J, LCM,
     $                   LCMQ, LDAA, LLWORK, LMAXWORK, LMIDWORK,
     $                   LMINWORK, LWMIN, MYCOL, MYROW, NEIG, NGEN,
     $                   NLOC, NN, NNODES, NP, NP0, NQ, NQ0, PTR2D,
     $                   PTR2WORK, RES, RSRC_A, SIZECHK, SIZEMQRLEFT,
     $                   SIZEMQRRIGHT, SIZEQRF, SIZEQTQ, SIZESYEVX,
     $                   SIZETMS
      DOUBLE PRECISION   ANINV, ANORM, COND, MAXQTQNRM, MAXTSTNRM, OVFL,
     $                   QTQNRM, RTOVFL, RTUNFL, TEMP1, TSTNRM, ULP,
     $                   ULPINV, UNFL, VL, VU
*     ..
*     .. Local Arrays ..
      INTEGER            DESCA( 8 ), ISEEDIN( 4 ), KMAGN( MAXTYP ),
     $                   KMODE( MAXTYP ), KTYPE( MAXTYP )
      DOUBLE PRECISION   CTIME( 10 ), WTIME( 10 )
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, ILCM, INDXG2P, NUMROC
      DOUBLE PRECISION   DLARAN, PDLAMCH
      EXTERNAL           LSAME, ICEIL, ILCM, INDXG2P, NUMROC, DLARAN,
     $                   PDLAMCH
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GET, BLACS_GRIDEXIT, BLACS_GRIDINFO,
     $                   BLACS_GRIDINIT, BLACS_PINFO, DLABAD, DLASRT,
     $                   DLATMS, IGAMX2D, IGEBR2D, IGEBS2D, PDELSET,
     $                   PDLASET, PDLATMS, PDMATGEN, PDSEPTST2,
     $                   SLCOMBINE
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, DBLE, INT, MAX, MIN, SQRT
*     ..
*     .. Data statements ..
      DATA               KTYPE / 1, 2, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 8,
     $                   8, 8, 9, 9, 9, 9, 9, 10, 11 /
      DATA               KMAGN / 1, 1, 1, 1, 1, 2, 3, 1, 1, 1, 2, 3, 1,
     $                   2, 3, 1, 1, 1, 2, 3, 1, 1 /
      DATA               KMODE / 0, 0, 4, 3, 1, 4, 4, 4, 3, 1, 4, 4, 0,
     $                   0, 0, 4, 3, 1, 4, 4, 3, 0 /
*     ..
*     .. Executable Statements ..
*
      CALL BLACS_PINFO( IAM, NNODES )
      CALL BLACS_GET( -1, 0, CONTEXT )
      CALL BLACS_GRIDINIT( CONTEXT, 'R', NPROW, NPCOL )
      CALL BLACS_GRIDINFO( CONTEXT, NPROW, NPCOL, MYROW, MYCOL )
*
      RESULT = 0
*
*     If this process is not a part of the contex, bail out now.
*
      IF( ( MYROW.GE.NPROW ) .OR. ( MYROW.LT.0 ) .OR.
     $    ( MYCOL.GE.NPCOL ) .OR. ( MYCOL.LT.0 ) )GO TO 140
*
*     Make sure that we have enough memory
*
      DESCA( 1 ) = N
      DESCA( 2 ) = N
      DESCA( 3 ) = NB
      DESCA( 4 ) = NB
      DESCA( 5 ) = 0
      DESCA( 6 ) = 0
      DESCA( 7 ) = CONTEXT
      DESCA( 8 ) = LDA
*
      LDAA = DESCA( 8 )
      NB = DESCA( 3 )
      RSRC_A = DESCA( 5 )
      CSRC_A = DESCA( 6 )
      LCM = ILCM( NPROW, NPCOL )
      LCMQ = LCM / NPCOL
      IROFFA = 0
      ICOFFA = 0
      IAROW = INDXG2P( 1, NB, MYROW, RSRC_A, NPROW )
      IACOL = INDXG2P( 1, NB, MYCOL, CSRC_A, NPCOL )
      NP = NUMROC( N+IROFFA, NB, MYROW, IAROW, NPROW )
      NQ = NUMROC( N+ICOFFA, NB, MYCOL, IACOL, NPCOL )
      SIZEMQRLEFT = MAX( ( NB*( NB-1 ) ) / 2, ( NP+NQ )*NB ) +
     $              ( NP+NB )*NB
      SIZEMQRRIGHT = MAX( ( NB*( NB-1 ) ) / 2,
     $               ( NQ+MAX( NP+NUMROC( NUMROC( N+ICOFFA, NB, 0, 0,
     $               NPCOL ), NB, 0, 0, LCMQ ), NP ) )*NB ) + NB*NB
      SIZEQRF = NB*NP + NB*NQ + NB*NB
      SIZETMS = ( LDAA+1 )*MAX( 1, NQ ) + 200 +
     $          MAX( SIZEMQRLEFT, SIZEMQRRIGHT, SIZEQRF )
*
      NP0 = NUMROC( N, DESCA( 3 ), 0, 0, NPROW )
      NQ0 = NUMROC( N, DESCA( 4 ), 0, 0, NPCOL )
      SIZEQTQ = 2 + MAX( DESCA( 3 ), 2 )*( 2*NP0+NQ0 )
*
      SIZECHK = NUMROC( N, DESCA( 4 ), MYCOL, 0, NPCOL )
*
      NEIG = N
      NN = MAX( N, NB, 2 )
      NP0 = NUMROC( NN, NB, 0, 0, NPROW )
      NQ0 = MAX( NUMROC( NEIG, NB, 0, 0, NPCOL ), NB )
      SIZESYEVX = 5*N + MAX( 5*NN, NP0*NQ0 ) +
     $            ICEIL( NEIG, NPROW*NPCOL )*NN + 2*NB*NB
*
      LWMIN = N + MAX( SIZETMS, SIZEQTQ, SIZECHK, SIZESYEVX )
*
      CALL IGAMX2D( CONTEXT, 'a', ' ', 1, 1, LWMIN, 1, 1, 1, -1, -1, 0 )
*
      SIZESYEVX = 5*N + MAX( 5*NN, NP0*NQ0 ) + NEIG*NN + 2*NB*NB
*
      LMAXWORK = MAX( N+SIZESYEVX, LWMIN )
      IF( LWORK.LT.LWMIN ) THEN
         RESULT = 99
      END IF
*
      CALL IGAMX2D( CONTEXT, 'a', ' ', 1, 1, RESULT, 1, 1, 1, -1, -1,
     $              0 )
*
      IF( RESULT.NE.0 ) THEN
         IF( IAM.EQ.0 ) THEN
            WRITE( NOUT, FMT = 9989 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES
            WRITE( NOUT, FMT = 9975 )
         END IF
         RESULT = 0
         GO TO 130
      END IF
*
*
*
*
*
      PTR2D = 1
      PTR2WORK = PTR2D + N
      LLWORK = LWORK - PTR2WORK + 1
*
      ULP = PDLAMCH( CONTEXT, 'P' )
      ULPINV = ONE / ULP
      UNFL = PDLAMCH( CONTEXT, 'Safe min' )
      OVFL = ONE / UNFL
      CALL DLABAD( UNFL, OVFL )
      RTUNFL = SQRT( UNFL )
      RTOVFL = SQRT( OVFL )
      ANINV = ONE / DBLE( MAX( 1, N ) )
*
*     This ensures that everyone starts out with the same seed.
*
      IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 ) THEN
         CALL IGEBS2D( CONTEXT, 'a', ' ', 4, 1, ISEED, 4 )
      ELSE
         CALL IGEBR2D( CONTEXT, 'a', ' ', 4, 1, ISEED, 4, 0, 0 )
      END IF
      ISEEDIN( 1 ) = ISEED( 1 )
      ISEEDIN( 2 ) = ISEED( 2 )
      ISEEDIN( 3 ) = ISEED( 3 )
      ISEEDIN( 4 ) = ISEED( 4 )
*
*     Compute the matrix A
*
*     Control parameters:
*
*         KMAGN  KMODE        KTYPE
*     =1  O(1)   clustered 1  zero
*     =2  large  clustered 2  identity
*     =3  small  exponential  (none)
*     =4         arithmetic   diagonal, (w/ eigenvalues)
*     =5         random log   symmetric, w/ eigenvalues
*     =6         random       (none)
*     =7                      random diagonal
*     =8                      random symmetric
*     =9                      positive definite
*     =10                     diagonally dominant tridiagonal
*
      ITYPE = KTYPE( MATTYPE )
      IMODE = KMODE( MATTYPE )
*
*           Compute norm
*
      GO TO ( 10, 20, 30 )KMAGN( MATTYPE )
*
   10 CONTINUE
      ANORM = ONE
      GO TO 40
*
   20 CONTINUE
      ANORM = ( RTOVFL*ULP )*ANINV
      GO TO 40
*
   30 CONTINUE
      ANORM = RTUNFL*N*ULPINV
      GO TO 40
*
   40 CONTINUE
*
*     IF (MATTYPE .EQ. 1) THEN
*        COND = 100.0
*        ANORM = 1.0
*        IMODE = 4
*     ELSE IF (MATTYPE .EQ. 2) THEN
*        COND = 1/ULP
*        ANORM = 1.0
*        IMODE = 1
*     ENDIF
*
*
      IF( MATTYPE.LE.15 ) THEN
         COND = ULPINV
      ELSE
         COND = ULPINV*ANINV / TEN
      END IF
*
*     Special Matrices
*
*     Zero
*
*
      IF( ITYPE.EQ.1 ) THEN
*
*        Zero Matrix
*
         DO 50 I = 1, N
            WORK( PTR2D+I-1 ) = ZERO
   50    CONTINUE
         CALL PDLASET( 'All', N, N, ZERO, ZERO, COPYA, 1, 1, DESCA )
*
      ELSE IF( ITYPE.EQ.2 ) THEN
*
*        Identity Matrix
*
         DO 60 I = 1, N
            WORK( PTR2D+I-1 ) = ONE
   60    CONTINUE
         CALL PDLASET( 'All', N, N, ZERO, ONE, COPYA, 1, 1, DESCA )
*
      ELSE IF( ITYPE.EQ.4 ) THEN
*
*        Diagonal Matrix, [Eigen]values Specified
*
         CALL PDLATMS( N, N, 'S', ISEED, 'S', WORK( PTR2D ), IMODE,
     $                 COND, ANORM, 0, 0, 'N', COPYA, 1, 1, DESCA,
     $                 ORDER, WORK( PTR2WORK ), LLWORK, INFO )
*
      ELSE IF( ITYPE.EQ.5 ) THEN
*
*        Symmetric, eigenvalues specified
*
         CALL PDLATMS( N, N, 'S', ISEED, 'S', WORK( PTR2D ), IMODE,
     $                 COND, ANORM, N, N, 'N', COPYA, 1, 1, DESCA,
     $                 ORDER, WORK( PTR2WORK ), LLWORK, INFO )
*
      ELSE IF( ITYPE.EQ.8 ) THEN
*
*        Symmetric, random eigenvalues
*
         NP = NUMROC( N, DESCA( 3 ), MYROW, 0, NPROW )
         NQ = NUMROC( N, DESCA( 4 ), MYCOL, 0, NPCOL )
         CALL PDMATGEN( DESCA( 7 ), 'S', 'N', N, N, DESCA( 3 ),
     $                  DESCA( 4 ), COPYA, DESCA( 8 ), DESCA( 5 ),
     $                  DESCA( 6 ), ISEED( 1 ), 0, NP, 0, NQ, MYROW,
     $                  MYCOL, NPROW, NPCOL )
         INFO = 0
*
      ELSE IF( ITYPE.EQ.9 ) THEN
*
*        Positive definite, eigenvalues specified.
*
         CALL PDLATMS( N, N, 'S', ISEED, 'P', WORK( PTR2D ), IMODE,
     $                 COND, ANORM, N, N, 'N', COPYA, 1, 1, DESCA,
     $                 ORDER, WORK( PTR2WORK ), LLWORK, INFO )
*
      ELSE IF( ITYPE.EQ.10 ) THEN
*
*        Positive definite tridiagonal submatrices, eigenvalues spec'd.
*
         CALL PDLASET( 'All', N, N, ZERO, ZERO, COPYA, 1, 1, DESCA )
         NP = NUMROC( N, DESCA( 3 ), 0, 0, NPROW )
         NQ = NUMROC( N, DESCA( 4 ), 0, 0, NPCOL )
         NLOC = MIN( NP, NQ )
         NGEN = 0
   70    CONTINUE
*
         IF( NGEN.LT.N ) THEN
            IN = MIN( 1+INT( DLARAN( ISEED )*DBLE( NLOC ) ), N-NGEN )
            CALL DLATMS( IN, IN, 'S', ISEED, 'P', WORK( PTR2D ), IMODE,
     $                   COND, ANORM, 1, 1, 'N', A, LDA,
     $                   WORK( PTR2WORK ), INFO )
            DO 80 I = 2, IN
               TEMP1 = ABS( A( I-1, I ) ) /
     $                 SQRT( ABS( A( I-1, I-1 )*A( I, I ) ) )
               IF( TEMP1.GT.HALF ) THEN
                  A( I-1, I ) = HALF*SQRT( ABS( A( I-1, I-1 )*A( I,
     $                          I ) ) )
                  A( I, I-1 ) = A( I-1, I )
               END IF
   80       CONTINUE
            CALL PDELSET( COPYA, NGEN+1, NGEN+1, DESCA, A( 1, 1 ) )
            DO 90 I = 2, IN
               CALL PDELSET( COPYA, NGEN+I, NGEN+I, DESCA, A( I, I ) )
               CALL PDELSET( COPYA, NGEN+I-1, NGEN+I, DESCA,
     $                       A( I-1, I ) )
               CALL PDELSET( COPYA, NGEN+I, NGEN+I-1, DESCA,
     $                       A( I, I-1 ) )
   90       CONTINUE
            NGEN = NGEN + IN
            GO TO 70
         END IF
*
      ELSE IF( ITYPE.EQ.11 ) THEN
*
*        Symmetric, eigenvalues specified
*
         NGEN = 0
         J = 1
         TEMP1 = ZERO
  100    CONTINUE
         IF( NGEN.LT.N ) THEN
            IN = MIN( J, N-NGEN )
            DO 110 I = 0, IN - 1
               WORK( PTR2D+NGEN+I ) = TEMP1
  110       CONTINUE
            TEMP1 = TEMP1 + ONE
            J = 2*J
            NGEN = NGEN + IN
            GO TO 100
         END IF
*
*
         CALL PDLATMS( N, N, 'S', ISEED, 'S', WORK( PTR2D ), IMODE,
     $                 COND, ANORM, N, N, 'N', COPYA, 1, 1, DESCA,
     $                 ORDER, WORK( PTR2WORK ), LLWORK, INFO )
*
      ELSE
         INFO = 1
      END IF
*
      CALL PDSEPTST2( 'v', 'a', UPLO, N, VL, VU, IL, IU, THRESH, ABSTOL,
     $                A, COPYA, Z, 1, 1, DESCA, WIN, WNEW, IFAIL,
     $                ICLUSTR, GAP, IPREPAD, IPOSTPAD, WORK( PTR2WORK ),
     $                LLWORK, IWORK, LIWORK, RES, TSTNRM, QTQNRM, NOUT )
*
      MAXTSTNRM = TSTNRM
      MAXQTQNRM = QTQNRM
*
*     Here we check to make sure that the matrix that we generated
*     had the eigenvalues that we expected.
*
      IF( THRESH.GT.0 ) THEN
         IF( ITYPE.NE.8 .AND. ITYPE.NE.10 ) THEN
            CALL DLASRT( 'I', N, WORK( PTR2D ), INFO )
*
            IF( RES.EQ.0 ) THEN
               DO 120 I = 1, N
                  IF( ABS( WORK( PTR2D+I-1 )-WIN( I ) ).GT.100D0*ULP*
     $                MAX( UNFL / ULP, ABS( WORK( PTR2D ) ),
     $                ABS( WORK( PTR2D+N-1 ) ) ) ) THEN
                     IF( IAM.EQ.0 )
     $                  WRITE( NOUT, FMT = 9988 )
     $                  'pdseptst.f TEST ERROR I=', I, ' W(I)=',
     $                  WIN( I ), ' should be:', WORK( PTR2D+I-1 ),
     $                  ' diff=', ABS( WIN( I )-WORK( PTR2D+I-1 ) ),
     $                  ' should be less than:',
     $                  100.0D0*ULP*MAX( UNFL / ULP,
     $                  ABS( WORK( PTR2D ) ), ABS( WORK( PTR2D+N-1 ) ) )
     $                  , ' ratio is:', ABS( WIN( I )-
     $                  WORK( PTR2D+I-1 ) ) / ( 100.0D0*ULP*
     $                  MAX( UNFL / ULP, ABS( WORK( PTR2D ) ),
     $                  ABS( WORK( PTR2D+N-1 ) ) ) )
*
                     RESULT = 1
                  END IF
  120          CONTINUE
            END IF
         END IF
      END IF
*
      IF( RES.NE.0 ) THEN
         RESULT = 1
      END IF
*
      IF( NEIGTYPES.NE.1 ) THEN
         IF( RESULT.EQ.0 ) THEN
*
*           Make sure that we can compute just the eigenvalues
*
            CALL PDSEPTST2( 'n', 'a', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LLWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( RES.NE.0 ) THEN
               RESULT = 2
            END IF
         END IF
*
         IF( RESULT.EQ.0 ) THEN
*
*           Test PDSYEVX (values only) on a range of E-values (by index)
*
            IL = 1 + INT( DLARAN( ISEED )*DBLE( N ) )
            IU = IL + INT( DLARAN( ISEED )*DBLE( N-IL+1 ) )
            IF( N.EQ.0 )
     $         IU = 0
*
*
            CALL PDSEPTST2( 'n', 'i', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LLWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( RES.NE.0 ) THEN
               RESULT = 11
            END IF
         END IF
*
         IF( RESULT.EQ.0 ) THEN
*
*           Test PDSYEVX (vecs also) on a range of E-values (by index)
*
            IL = 1 + INT( DLARAN( ISEED )*DBLE( N ) )
            IU = IL + INT( DLARAN( ISEED )*DBLE( N-IL+1 ) )
            IF( N.EQ.0 )
     $         IU = 0
*
*
            CALL PDSEPTST2( 'v', 'i', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LLWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( RES.NE.0 ) THEN
               RESULT = 12
            END IF
         END IF
*
         IF( RESULT.EQ.0 ) THEN
*
*           Test PDSYEVX (vals only) on a range of E-values (by value)
*
            IL = 1 + INT( DLARAN( ISEED )*DBLE( N ) )
            IU = IL + INT( DLARAN( ISEED )*DBLE( N-IL+1 ) )
            IF( WIN( IL ).LE.ZERO ) THEN
               VL = WIN( IL )*( ONE+TWO*ULP*DLARAN( ISEED ) ) - UNFL
            ELSE
               VL = WIN( IL )*( ONE-TWO*ULP*DLARAN( ISEED ) )
            END IF
            IF( WIN( IU ).LE.ZERO ) THEN
               VU = WIN( IU )*( ONE-TWO*ULP*DLARAN( ISEED ) ) + UNFL
            ELSE
               VU = WIN( IU )*( ONE+TWO*ULP*DLARAN( ISEED ) )
            END IF
            IF( VU.EQ.VL )
     $         VU = VU + ABS( VU )*TWO*ULP + UNFL
            IF( VU.LE.VL ) THEN
               WRITE( NOUT, FMT = 9988 )' pdseptst:: VL <= VU'
            END IF
*
            CALL PDSEPTST2( 'n', 'v', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LLWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( RES.NE.0 ) THEN
               RESULT = 21
            END IF
         END IF
*
         IF( RESULT.EQ.0 ) THEN
*
*           Test PDSYEVX (vecs also) on a range of E-values (by value)
*
            IL = 1 + INT( DLARAN( ISEED )*DBLE( N ) )
            IU = IL + INT( DLARAN( ISEED )*DBLE( N-IL+1 ) )
            IF( WIN( IL ).LE.ZERO ) THEN
               VL = WIN( IL )*( ONE+TWO*ULP*DLARAN( ISEED ) ) - UNFL
            ELSE
               VL = WIN( IL )*( ONE-TWO*ULP*DLARAN( ISEED ) )
            END IF
            IF( WIN( IU ).LE.ZERO ) THEN
               VU = WIN( IU )*( ONE-TWO*ULP*DLARAN( ISEED ) ) + UNFL
            ELSE
               VU = WIN( IU )*( ONE+TWO*ULP*DLARAN( ISEED ) )
            END IF
            IF( VU.EQ.VL )
     $         VU = VU + ABS( VU )*TWO*ULP + UNFL
            IF( VU.LE.VL ) THEN
               WRITE( NOUT, FMT = 9988 )' pdseptst:: VL <= VU'
            END IF
*
            IL = -13
            IU = -15
*
            CALL PDSEPTST2( 'v', 'v', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LLWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( ( IL.NE.-13 ) .OR. ( IU.NE.-15 ) ) THEN
               WRITE( NOUT, FMT = 9988 )
     $            'IL or IU should not be changed'
               RESULT = 22
            END IF
*
            IF( RES.NE.0 ) THEN
               RESULT = 22
            END IF
         END IF
         IF( RESULT.EQ.0 ) THEN
*
*           Test PDSYEVX with minimum memory
*
            NEIG = N
            NN = MAX( N, NB, 2 )
            NB = DESCA( 4 )
            NP0 = NUMROC( NN, NB, 0, 0, NPROW )
            NQ0 = MAX( NUMROC( NEIG, NB, 0, 0, NPCOL ), NB )
*
            SIZESYEVX = 5*N + MAX( 5*NN, NP0*NQ0 ) +
     $                  ICEIL( NEIG, NPROW*NPCOL )*NN + 2*NB*NB
*
*
            LMINWORK = LWMIN
*
            CALL PDSEPTST2( 'v', 'a', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LMINWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( RES.NE.0 ) THEN
               RESULT = 41
            END IF
         END IF
         IF( RESULT.EQ.0 ) THEN
*
*           Test PDSYEVX with more than minimum, less than max memory
*
            LMIDWORK = LMINWORK + INT( DLARAN( ISEED )*
     $                 ( LMAXWORK-LMINWORK ) )
*
            CALL PDSEPTST2( 'v', 'a', UPLO, N, VL, VU, IL, IU, THRESH,
     $                      ABSTOL, A, COPYA, Z, 1, 1, DESCA, WIN, WNEW,
     $                      IFAIL, ICLUSTR, GAP, IPREPAD, IPOSTPAD,
     $                      WORK( PTR2WORK ), LMIDWORK, IWORK, LIWORK,
     $                      RES, TSTNRM, QTQNRM, NOUT )
*
            MAXTSTNRM = MAX( TSTNRM, MAXTSTNRM )
            MAXQTQNRM = MAX( QTQNRM, MAXQTQNRM )
*
            IF( RES.NE.0 ) THEN
               RESULT = 42
            END IF
         END IF
      END IF
*
*
*
      CALL IGAMX2D( CONTEXT, 'All', ' ', 1, 1, RESULT, 1, -1, -1, -1,
     $              -1, -1 )
*
      IF( .FALSE. .OR. RESULT.NE.0 ) THEN
         IF( IAM.EQ.0 ) THEN
            WRITE( NOUT, FMT = 9988 )'C  '
            WRITE( NOUT, FMT = 9987 )ISEEDIN( 1 )
            WRITE( NOUT, FMT = 9986 )ISEEDIN( 2 )
            WRITE( NOUT, FMT = 9985 )ISEEDIN( 3 )
            WRITE( NOUT, FMT = 9984 )ISEEDIN( 4 )
            IF( LSAME( UPLO, 'L' ) ) THEN
               WRITE( NOUT, FMT = 9988 )'      UPLO= ''L'' '
            ELSE
               WRITE( NOUT, FMT = 9988 )'      UPLO= ''U'' '
            END IF
            WRITE( NOUT, FMT = 9983 )N
            WRITE( NOUT, FMT = 9982 )NPROW
            WRITE( NOUT, FMT = 9981 )NPCOL
            WRITE( NOUT, FMT = 9980 )NB
            WRITE( NOUT, FMT = 9979 )MATTYPE
            WRITE( NOUT, FMT = 9978 )NEIGTYPES
            WRITE( NOUT, FMT = 9977 )ABSTOL
            WRITE( NOUT, FMT = 9976 )THRESH
            WRITE( NOUT, FMT = 9988 )'C  '
         END IF
      END IF
*
      CALL SLCOMBINE( CONTEXT, 'All', '>', 'W', 6, 1, WTIME )
      CALL SLCOMBINE( CONTEXT, 'All', '>', 'C', 6, 1, CTIME )
      IF( IAM.EQ.0 ) THEN
         IF( PRINTREQ.EQ.1 .AND. RESULT.EQ.0 ) THEN
            IF( WTIME( 1 ).GE.0.0 ) THEN
               WRITE( NOUT, FMT = 9998 )WTIME( 1 ), CTIME( 1 ), N,
     $            NPROW, NPCOL, NB, MATTYPE, NEIGTYPES, MAXTSTNRM,
     $            MAXQTQNRM
            ELSE
               WRITE( NOUT, FMT = 9999 )CTIME( 1 ), N, NPROW, NPCOL,
     $            NB, MATTYPE, NEIGTYPES, MAXTSTNRM, MAXQTQNRM
            END IF
         END IF
         IF( RESULT.EQ.1 ) THEN
            WRITE( NOUT, FMT = 9997 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.2 ) THEN
            WRITE( NOUT, FMT = 9996 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.11 ) THEN
            WRITE( NOUT, FMT = 9995 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.12 ) THEN
            WRITE( NOUT, FMT = 9994 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.21 ) THEN
            WRITE( NOUT, FMT = 9993 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.22 ) THEN
            WRITE( NOUT, FMT = 9992 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.41 ) THEN
            WRITE( NOUT, FMT = 9991 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
         IF( RESULT.EQ.42 ) THEN
            WRITE( NOUT, FMT = 9990 )N, NPROW, NPCOL, NB, MATTYPE,
     $         NEIGTYPES, MAXTSTNRM, MAXQTQNRM
         END IF
      END IF
*
  130 CONTINUE
*
      CALL BLACS_GRIDEXIT( CONTEXT )
  140 CONTINUE
*
*
      RETURN
 9999 FORMAT( ' Passed           ', F9.2, I7, I5, I5, I5, I3, I3, G9.2,
     $      G9.2 )
 9998 FORMAT( ' Passed  ', F9.2, F9.2, I7, I5, I5, I5, I3, I3, G9.2,
     $      G9.2 )
 9997 FORMAT( ' Failed                    ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9996 FORMAT( ' Failed evals              ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9995 FORMAT( ' Failed subset a           ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9994 FORMAT( ' Failed subset b           ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9993 FORMAT( ' Failed subset c           ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9992 FORMAT( ' Failed subset             ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9991 FORMAT( ' Failed memory a           ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9990 FORMAT( ' Failed memory b           ', I7, I5, I5, I5, I3, I3,
     $      G9.2, G9.2 )
 9989 FORMAT( ' Too little memory         ', I7, I5, I5, I5, I3, I3 )
 9988 FORMAT( A )
 9987 FORMAT( '      ISEED( 1 ) =', I8 )
 9986 FORMAT( '      ISEED( 2 ) =', I8 )
 9985 FORMAT( '      ISEED( 3 ) =', I8 )
 9984 FORMAT( '      ISEED( 4 ) =', I8 )
 9983 FORMAT( '      N=', I8 )
 9982 FORMAT( '      NPROW=', I8 )
 9981 FORMAT( '      NPCOL=', I8 )
 9980 FORMAT( '      NB=', I8 )
 9979 FORMAT( '      MATTYPE=', I8 )
 9978 FORMAT( '      NEIGTYPES=', I8 )
 9977 FORMAT( '      ABSTOL=', D10.2 )
 9976 FORMAT( '      THRESH=', D10.2 )
 9975 FORMAT( ' Increase TOTMEM in pdsepdriver.f' )
*
      END
