      SUBROUTINE PDSEPTST2( JOBZ, RANGE, UPLO, N, VL, VU, IL, IU,
     $                      THRESH, ABSTOL, A, COPYA, Z, IA, JA, DESCA,
     $                      WIN, WNEW, IFAIL, ICLUSTR, GAP, IPREPAD,
     $                      IPOSTPAD, WORK, LWORK, IWORK, LIWORK,
     $                      RESULT, TSTNRM, QTQNRM, NOUT )
*
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      CHARACTER          JOBZ, RANGE, UPLO
      INTEGER            IA, IL, IPOSTPAD, IPREPAD, IU, JA, LIWORK,
     $                   LWORK, N, NOUT, RESULT
      DOUBLE PRECISION   ABSTOL, QTQNRM, THRESH, TSTNRM, VL, VU
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( 8 ), ICLUSTR( * ), IFAIL( * ),
     $                   IWORK( * )
      DOUBLE PRECISION   A( * ), COPYA( * ), GAP( * ), WIN( * ),
     $                   WNEW( * ), WORK( * ), Z( * )
*     ..
*
*  Purpose
*  =======
*
*  PDSEPTST2 calls PDSYEVX and then tests the output of PDSYEVX
*  If JOBZ = 'V' then the following two tests are performed:
*     |AQ -QL| / (abstol + eps * norm(A) ) < THRESH
*     |QT * Q - I| / eps * norm(A) < THRESH
*  If JOBZ = 'N' or RANGE <> 'A' then
*     we check to make sure that the eigenvalues match expectations
*     i.e. |WIN - WNEW(1+IPREPAD)| / (eps * |WIN|) < THRESH
*     where WIN is the array of eigenvalues as computed by
*     PDSYEVX when eigenvectors are requested
*
*  The first call to PDSEPTST2 must be made with JOBZ='V' and
*  RANGE = 'A'.  This will then set W() so that it can be used as
*  the standard for future comparison.
*
*  Arguments
*  =========
*
*     NP = the number of rows local to a given process.
*     NQ = the number of columns local to a given process.
*
*  JOBZ    (global input) CHARACTER*1
*          Specifies whether or not to compute the eigenvectors:
*          = 'N':  Compute eigenvalues only.
*          = 'V':  Compute eigenvalues and eigenvectors.
*          Must be 'V' on first call to PDSEPTST2
*
*  RANGE   (global input) CHARACTER*1
*          = 'A': all eigenvalues will be found.
*          = 'V': all eigenvalues in the half-open interval (VL,VU]
*                 will be found.
*          = 'I': the IL-th through IU-th eigenvalues will be found.
*          Must be 'A' on first call to PDSEPTST2
*
*  UPLO    (global input) CHARACTER*1
*          Specifies whether the upper or lower triangular part of the
*          symmetric matrix A is stored:
*          = 'U':  Upper triangular
*          = 'L':  Lower triangular
*
*  N       (global input) INTEGER
*          Size of the matrix to be tested.  (global size)
*
*  VL      (global input) DOUBLE PRECISION
*          If RANGE='V', the lower bound of the interval to be searched
*          for eigenvalues.  Not referenced if RANGE = 'A' or 'I'.
*
*  VU      (global input) DOUBLE PRECISION
*          If RANGE='V', the upper bound of the interval to be searched
*          for eigenvalues.  Not referenced if RANGE = 'A' or 'I'.
*
*  IL      (global input) INTEGER
*          If RANGE='I', the index (from smallest to largest) of the
*          smallest eigenvalue to be returned.  IL >= 1.
*          Not referenced if RANGE = 'A' or 'V'.
*
*  IU      (global input) INTEGER
*          If RANGE='I', the index (from smallest to largest) of the
*          largest eigenvalue to be returned.  min(IL,N) <= IU <= N.
*          Not referenced if RANGE = 'A' or 'V'.
*
*  THRESH  (global input) DOUBLE PRECISION
*          A test will count as "failed" if the "error", computed as
*          described below, exceeds THRESH.  Note that the error
*          is scaled to be O(1), so THRESH should be a reasonably
*          small multiple of 1, e.g., 10 or 100.  In particular,
*          it should not depend on the precision (single vs. double)
*          or the size of the matrix.  It must be at least zero.
*
*  ABSTOL  (global input) DOUBLE PRECISION
*          The absolute tolerance for the eigenvalues. An
*          eigenvalue is considered to be located if it has
*          been determined to lie in an interval whose width
*          is "abstol" or less. If "abstol" is less than or equal
*          to zero, then ulp*|T| will be used, where |T| is
*          the 1-norm of the matrix. If eigenvectors are
*          desired later by inverse iteration ("PDSTEIN"),
*          "abstol" MUST NOT be bigger than ulp*|T|.
*
*  A       (local workspace) DOUBLE PRECISION array
*          global dimension (N, N), local dimension (DESCA(8), NQ)
*            A is distributed in a block cyclic manner over both rows
*            and columns.
*            See pdsyevx.f for a description of block cyclic layout.
*          The test matrix, which is then modified by PDSYEVX
*          A has already been padded front and back, use A(1+IPREPAD)
*
*  COPYA   (local input) DOUBLE PRECISION array, dimension(N*N)
*          COPYA holds a copy of the original matrix A
*          identical in both form and content to A
*
*  Z       (local workspace) DOUBLE PRECISION array, dim (N*N)
*          Z is distributed in the same manner as A
*          Z contains the eigenvector matrix
*          Z is used as workspace by the test routines
*          pdsepchk,f and pdsepqtq.f.
*          Z has already been padded front and back, use Z(1+IPREPAD)
*
*  IA      (global input) INTEGER
*          On entry, IA specifies the global row index of the submatrix
*          of the global matrix A, COPYA and Z to operate on.
*
*  JA      (global input) INTEGER
*          On entry, IA specifies the global column index of the submat
*          of the global matrix A, COPYA and Z to operate on.
*
*  DESCA   (global/local input) INTEGER array of dimension 8
*          The array descriptor for the matrix A, COPYA and Z.
*
*  WIN     (global input/output) DOUBLE PRECISION array, dimension (N)
*          If JOBZ = 'V' and RANGE = 'A' then WIN is ignored on input
*          and set to the eigenvalues as computed by PDSYEVX on output
*          Otherwise, WIN() is taken as the standard by which the
*          eigenvalues are to be compared against.
*
*  WNEW    (global workspace)  DOUBLE PRECISION array, dimension (N)
*          The eigenvalues as copmuted by this call to PDSYEVX
*          If JOBZ <> 'V' or RANGE <> 'A' these eigenvalues are
*          compared against those in WIN().
*          WNEW has already been padded front and back,
*          use WNEW(1+IPREPAD)
*
*  IFAIL   (global output) INTEGER array, dimension (N)
*          If JOBZ = 'V', then on normal exit, the first M elements of
*          IFAIL are zero.  If INFO > 0 on exit, then IFAIL contains the
*          indices of the eigenvectors that failed to converge.
*          If JOBZ = 'N', then IFAIL is not referenced.
*          IFAIL has already been padded front and back,
*          use IFAIL(1+IPREPAD)
*
*  ICLUSTR (global workspace) integer array, dimension (2*NPROW*NPCOL)
*
*  GAP     (global workspace) DOUBLE PRECISION array,
*          dimension (NPROW*NPCOL)
*
*  WORK    (local workspace) DOUBLE PRECISION array, dimension (LWORK)
*          WORK has already been padded front and back,
*          use WORK(1+IPREPAD)
*
*  LWORK   (local input) INTEGER
*          The length of the array WORK after padding.
*
*  IWORK   (local workspace) INTEGER array, dimension (LIWORK)
*          IWORK has already been padded front and back,
*          use IWORK(1+IPREPAD)
*
*  LIWORK  (local input) INTEGER
*          The length of the array IWORK after padding.
*
*  RESULT  (global output) INTEGER
*          The result of this call to PDSYEVX
*          RESULT = -3   =>  This process did not participate
*          RESULT = 0    =>  All tests passed
*          RESULT = 1    =>  ONe or more tests failed
*
*  TSTNRM  (global output) DOUBLE PRECISION
*          |AQ- QL| / |A|*N*EPS
*
*  QTQNRM  (global output) DOUBLE PRECISION
*          |QTQ -I| / N*EPS
*
*     .. Parameters ..
*
      DOUBLE PRECISION   PADVAL, FIVE
      PARAMETER          ( PADVAL = 13.5285D0, FIVE = 5.0D0 )
      INTEGER            IPADVAL
      PARAMETER          ( IPADVAL = 927 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, IAM, INFO, J, M, MAXEIGS, MAXIL, MAXIU,
     $                   MAXSIZESYEVX, MINIL, MQ, MYCOL, MYIL, MYROW,
     $                   NB, NCLUSTERS, NEIG, NN, NP, NP0, NPCOL, NPROW,
     $                   NQ, NQ0, NZ, RES
      DOUBLE PRECISION   EPS, EPSNORMA, ERROR, MAXERROR, MAXVU,
     $                   MINERROR, MINVL, NORMWIN, ORFAC, SAFMIN
*     ..
*     .. Local Arrays ..
      INTEGER            DESCZ( 8 ), ITMP( 2 )
*     ..
*     .. External Functions ..
*
      LOGICAL            LSAME
      INTEGER            NUMROC
      DOUBLE PRECISION   PDLAMCH, PDLANSY
      EXTERNAL           LSAME, NUMROC, PDLAMCH, PDLANSY
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DESCSET, DGAMN2D, DGAMX2D,
     $                   DLACPY, IGAMN2D, IGAMX2D, PDCHEKPAD, PDELSET,
     $                   PDFILLPAD, PDSEPCHK, PDSEPQTQ, PDSYEVX,
     $                   PICHEKPAD, PIFILLPAD, SLBOOT, SLTIMER
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN
*     ..
*     .. Executable Statements ..
*
*     Make sure that we aren't using information from previous calls
*
      EPS = PDLAMCH( DESCA( 7 ), 'Eps' )
      SAFMIN = PDLAMCH( DESCA( 7 ), 'Safe min' )
      NORMWIN = 0.0D0
      DO 10 I = 1, N
         NORMWIN = MAX( NORMWIN, ABS( WIN( I ) ) )
   10 CONTINUE
      NORMWIN = MAX( NORMWIN, SAFMIN / EPS )
*
      DO 20 I = 1, LWORK, 1
         WORK( I+IPREPAD ) = 14.3D0
   20 CONTINUE
      DO 30 I = 1, LIWORK, 1
         IWORK( I+IPREPAD ) = 14
   30 CONTINUE
*
      DO 40 I = 1, N
         WNEW( I+IPREPAD ) = 3.14159D0
   40 CONTINUE
*
      IF( LSAME( JOBZ, 'N' ) ) THEN
         MAXEIGS = 0
      ELSE
         IF( LSAME( RANGE, 'A' ) ) THEN
            MAXEIGS = N
         ELSE IF( LSAME( RANGE, 'I' ) ) THEN
            MAXEIGS = IU - IL + 1
         ELSE
            MINVL = VL - NORMWIN*2*EPS - ABSTOL
            MAXVU = VU + NORMWIN*2*EPS + ABSTOL
            MINIL = 1
            MAXIU = 0
            DO 50 I = 1, N
               IF( WIN( I ).LT.MINVL )
     $            MINIL = MINIL + 1
               IF( WIN( I ).LE.MAXVU )
     $            MAXIU = MAXIU + 1
   50       CONTINUE
*
            MAXEIGS = MAXIU - MINIL + 1
         END IF
      END IF
*
      CALL DESCSET( DESCZ, DESCA( 1 ), DESCA( 2 ), DESCA( 3 ),
     $              DESCA( 4 ), DESCA( 5 ), DESCA( 6 ), DESCA( 7 ),
     $              DESCA( 8 ) )
*
      CALL BLACS_GRIDINFO( DESCA( 7 ), NPROW, NPCOL, MYROW, MYCOL )
*
      IAM = 1
      IF( MYROW.EQ.0 .AND. MYCOL.EQ.0 )
     $   IAM = 0
*
*     If this process is not involved in this test, bail out now
*
      RESULT = -3
      IF( MYROW.GE.NPROW .OR. MYROW.LT.0 )
     $   GO TO 180
      RESULT = 0
*
      NB = DESCA( 3 )
      NN = MAX( N, NB, 2 )
      NP0 = NUMROC( NN, NB, 0, 0, NPROW )
      NEIG = N
      NQ0 = MAX( NUMROC( NEIG, NB, 0, 0, NPCOL ), NB )
      MAXSIZESYEVX = 5*N + MAX( 5*NN, NP0*NQ0 ) + N*NN + 2*NB*NB
*
      NP = NUMROC( N, DESCA( 3 ), MYROW, 0, NPROW )
      NQ = NUMROC( N, DESCA( 4 ), MYCOL, 0, NPCOL )
      MQ = NUMROC( MAXEIGS, DESCA( 4 ), MYCOL, 0, NPCOL )
*
      CALL DLACPY( 'A', NP, NQ, COPYA, DESCA( 8 ), A( 1+IPREPAD ),
     $             DESCA( 8 ) )
*
      CALL PDFILLPAD( DESCA( 7 ), NP, NQ, A, DESCA( 8 ), IPREPAD,
     $                IPOSTPAD, PADVAL )
*
      CALL PDFILLPAD( DESCZ( 7 ), NP, MQ, Z, DESCZ( 8 ), IPREPAD,
     $                IPOSTPAD, PADVAL+1.0D0 )
*
      CALL PDFILLPAD( DESCA( 7 ), N, 1, WNEW, N, IPREPAD, IPOSTPAD,
     $                PADVAL+2.0D0 )
*
      CALL PDFILLPAD( DESCA( 7 ), NPROW*NPCOL, 1, GAP, NPROW*NPCOL,
     $                IPREPAD, IPOSTPAD, PADVAL+3.0D0 )
*
      CALL PDFILLPAD( DESCA( 7 ), LWORK, 1, WORK, LWORK, IPREPAD,
     $                IPOSTPAD, PADVAL+4.0D0 )
*
      CALL PIFILLPAD( DESCA( 7 ), LIWORK, 1, IWORK, LIWORK, IPREPAD,
     $                IPOSTPAD, IPADVAL )
*
      CALL PIFILLPAD( DESCA( 7 ), N, 1, IFAIL, N, IPREPAD, IPOSTPAD,
     $                IPADVAL )
*
      CALL PIFILLPAD( DESCA( 7 ), 2*NPROW*NPCOL, 1, ICLUSTR,
     $                2*NPROW*NPCOL, IPREPAD, IPOSTPAD, IPADVAL )
*
*     Make sure that PDSYEVX does not cheat (i.e. use answers
*     already computed.)
*
      DO 70 I = 1, N, 1
         DO 60 J = 1, MAXEIGS, 1
            CALL PDELSET( Z( 1+IPREPAD ), I, J, DESCA, 13.0D0 )
   60    CONTINUE
   70 CONTINUE
*
      ORFAC = -1.0D0
*
      CALL SLBOOT
      CALL SLTIMER( 1 )
      CALL SLTIMER( 6 )
      CALL PDSYEVX( JOBZ, RANGE, UPLO, N, A( 1+IPREPAD ), IA, JA, DESCA,
     $              VL, VU, IL, IU, ABSTOL, M, NZ, WNEW( 1+IPREPAD ),
     $              ORFAC, Z( 1+IPREPAD ), IA, JA, DESCA,
     $              WORK( 1+IPREPAD ), LWORK, IWORK( 1+IPREPAD ),
     $              LIWORK-2, IFAIL( 1+IPREPAD ), ICLUSTR( 1+IPREPAD ),
     $              GAP( 1+IPREPAD ), INFO )
      CALL SLTIMER( 6 )
      CALL SLTIMER( 1 )
*
      IF( THRESH.GT.0 ) THEN
*
         CALL PDCHEKPAD( DESCA( 7 ), 'PDSYEVX-A', NP, NQ, A, DESCA( 8 ),
     $                   IPREPAD, IPOSTPAD, PADVAL )
*
         CALL PDCHEKPAD( DESCZ( 7 ), 'PDSYEVX-Z', NP, MQ, Z, DESCZ( 8 ),
     $                   IPREPAD, IPOSTPAD, PADVAL+1.0D0 )
*
         CALL PDCHEKPAD( DESCA( 7 ), 'PDSYEVX-WNEW', N, 1, WNEW, N,
     $                   IPREPAD, IPOSTPAD, PADVAL+2.0D0 )
*
         CALL PDCHEKPAD( DESCA( 7 ), 'PDSYEVX-GAP', NPROW*NPCOL, 1, GAP,
     $                   NPROW*NPCOL, IPREPAD, IPOSTPAD, PADVAL+3.0D0 )
*
         CALL PDCHEKPAD( DESCA( 7 ), 'PDSYEVX-WORK', LWORK, 1, WORK,
     $                   LWORK, IPREPAD, IPOSTPAD, PADVAL+4.0D0 )
*
         CALL PICHEKPAD( DESCA( 7 ), 'PDSYEVX-IWORK', LIWORK, 1, IWORK,
     $                   LIWORK, IPREPAD, IPOSTPAD, IPADVAL )
*
         CALL PICHEKPAD( DESCA( 7 ), 'PDSYEVX-IFAIL', N, 1, IFAIL, N,
     $                   IPREPAD, IPOSTPAD, IPADVAL )
*
         CALL PICHEKPAD( DESCA( 7 ), 'PDSYEVX-ICLUSTR', 2*NPROW*NPCOL,
     $                   1, ICLUSTR, 2*NPROW*NPCOL, IPREPAD, IPOSTPAD,
     $                   IPADVAL )
*
*     Check INFO
*
         IF( INFO.NE.0 .AND. INFO.NE.2 ) THEN
            WRITE( NOUT, FMT = 9999 )INFO
            RESULT = 1
         END IF
*
*     Make sure that all processes return the same value of INFO
*
         ITMP( 1 ) = INFO
         ITMP( 2 ) = INFO
*
         CALL IGAMN2D( DESCA( 7 ), 'a', ' ', 1, 1, ITMP, 1, 1, 1, -1,
     $                 -1, 0 )
         CALL IGAMX2D( DESCA( 7 ), 'a', ' ', 1, 1, ITMP( 2 ), 1, 1, 1,
     $                 -1, -1, 0 )
*
         IF( ITMP( 1 ).NE.ITMP( 2 ) ) THEN
            IF( IAM.EQ.0 )
     $         WRITE( NOUT, FMT = * )
     $         'Different processes return different INFO'
            RESULT = 1
         END IF
*
         IF( LWORK.GE.MAXSIZESYEVX ) THEN
            IF( INFO.NE.0 ) THEN
               IF( IAM.EQ.0 )
     $            WRITE( NOUT, FMT = * )
     $            ' INFO .NE. 0 despite enormous workspace'
               RESULT = 1
            END IF
            IF( LSAME( JOBZ, 'V' ) .AND. ( ICLUSTR( 1+IPREPAD ).NE.0 ) )
     $           THEN
               IF( IAM.EQ.0 )
     $            WRITE( NOUT, FMT = * )
     $            ' ICLUSTR(1).NE.0 despite enormous workspace'
               RESULT = 1
            END IF
         END IF
*
*     Check M
*
         IF( ( M.LT.0 ) .OR. ( M.GT.N ) ) THEN
            WRITE( NOUT, FMT = * )' M not in the range 0 to M'
            RESULT = 1
         ELSE IF( LSAME( RANGE, 'A' ) .AND. ( M.NE.N ) ) THEN
            WRITE( NOUT, FMT = * )' M not equal to N'
            RESULT = 1
         ELSE IF( LSAME( JOBZ, 'V' ) .AND. ( M.NE.NZ ) ) THEN
            WRITE( NOUT, FMT = * )' M not equal to NZ'
            RESULT = 1
         END IF
*
*     Make sure that all processes return the same # of eigenvalues
*
         ITMP( 1 ) = M
         ITMP( 2 ) = M
*
         CALL IGAMN2D( DESCA( 7 ), 'a', ' ', 1, 1, ITMP, 1, 1, 1, -1,
     $                 -1, 0 )
         CALL IGAMX2D( DESCA( 7 ), 'a', ' ', 1, 1, ITMP( 2 ), 1, 1, 1,
     $                 -1, -1, 0 )
*
         IF( ITMP( 1 ).NE.ITMP( 2 ) ) THEN
            IF( IAM.EQ.0 )
     $         WRITE( NOUT, FMT = * )
     $         'Different processes return different values for M'
            RESULT = 1
         END IF
*
*     Make sure that different processes return the same eigenvalues
*
         DO 80 I = 1, M
            WORK( I ) = WNEW( I+IPREPAD )
            WORK( I+M ) = WNEW( I+IPREPAD )
   80    CONTINUE
         CALL DGAMN2D( DESCA( 7 ), 'a', ' ', M, 1, WORK, M, 1, 1, -1,
     $                 -1, 0 )
         CALL DGAMX2D( DESCA( 7 ), 'a', ' ', M, 1, WORK( 1+M ), M, 1, 1,
     $                 -1, -1, 0 )
*
         DO 90 I = 1, M
            IF( RESULT.EQ.0 .AND. WORK( I ).NE.WORK( M+I ) ) THEN
               IF( IAM.EQ.0 )
     $            WRITE( NOUT, FMT = * )
     $            'Different processes return different Evalues'
               RESULT = 1
            END IF
   90    CONTINUE
*
*     Make sure that all processes return the same # of clusters
*
         NCLUSTERS = 0
         DO 100 I = 0, NPROW*NPCOL - 1
            IF( ICLUSTR( 1+IPREPAD+2*I ).EQ.0 )
     $         GO TO 110
            NCLUSTERS = NCLUSTERS + 1
  100    CONTINUE
  110    CONTINUE
         ITMP( 1 ) = NCLUSTERS
         ITMP( 2 ) = NCLUSTERS
*
         CALL IGAMN2D( DESCA( 7 ), 'a', ' ', 1, 1, ITMP, 1, 1, 1, -1,
     $                 -1, 0 )
         CALL IGAMX2D( DESCA( 7 ), 'a', ' ', 1, 1, ITMP( 2 ), 1, 1, 1,
     $                 -1, -1, 0 )
*
         IF( ITMP( 1 ).NE.ITMP( 2 ) ) THEN
            IF( IAM.EQ.0 )
     $         WRITE( NOUT, FMT = * )
     $         'Different processes return different number of clusters'
            RESULT = 1
         END IF
*
*     Make sure that different processes return the same clusters
*
         DO 120 I = 1, NCLUSTERS
            IWORK( I ) = ICLUSTR( I+IPREPAD )
            IWORK( I+NCLUSTERS ) = ICLUSTR( I+IPREPAD )
  120    CONTINUE
         CALL IGAMN2D( DESCA( 7 ), 'a', ' ', NCLUSTERS*2+1, 1,
     $                 IWORK( 1 ), NCLUSTERS*2+1, 1, 1, -1, -1, 0 )
         CALL IGAMX2D( DESCA( 7 ), 'a', ' ', NCLUSTERS*2+1, 1,
     $                 IWORK( 1+NCLUSTERS ), NCLUSTERS*2+1, 1, 1, -1,
     $                 -1, 0 )
*
*
         DO 130 I = 1, NCLUSTERS
            IF( RESULT.EQ.0 .AND. IWORK( I ).NE.IWORK( NCLUSTERS+I ) )
     $           THEN
               IF( IAM.EQ.0 )
     $            WRITE( NOUT, FMT = * )
     $            'Different processes return different clusters'
               RESULT = 1
            END IF
  130    CONTINUE
*
         DO 140 I = 1, M
            WORK( I ) = WNEW( I+IPREPAD )
            WORK( I+M ) = WNEW( I+IPREPAD )
  140    CONTINUE
*
         IF( RESULT.NE.0 )
     $      GO TO 180
*
*     Compute eps * norm(A)
*
         IF( N.EQ.0 ) THEN
            EPSNORMA = EPS
         ELSE
            EPSNORMA = PDLANSY( 'I', UPLO, N, COPYA, IA, JA, DESCA,
     $                 WORK )*EPS
         END IF
*
*     Note that a couple key variables get redefined in PDSEPCHK
*     as described by this table:
*
*     PDSEPTST name         PDSEPCHK name
*     -------------         -------------
*     COPYA                 A
*     Z                     Q
*     A                     B
*
*
         IF( LSAME( JOBZ, 'V' ) ) THEN
*
*     Perform the |AQ - QE| test
*
            CALL PDSEPCHK( N, M, COPYA, MAX( ABSTOL+EPSNORMA, SAFMIN ),
     $                     THRESH, Z( 1+IPREPAD ), IA, JA, DESCZ,
     $                     A( 1+IPREPAD ), IA, JA, DESCA,
     $                     WNEW( 1+IPREPAD ), WORK, LWORK, TSTNRM, RES )
*
            IF( RES.NE.0 )
     $         RESULT = 1
*
*     Perform the |QTQ - I| test
*
            CALL PDSEPQTQ( N, M, THRESH, Z( 1+IPREPAD ), IA, JA, DESCZ,
     $                     A( 1+IPREPAD ), IA, JA, DESCA,
     $                     IWORK( 1+IPREPAD+1 ), ICLUSTR( 1+IPREPAD ),
     $                     GAP( 1+IPREPAD ), WORK, LWORK, QTQNRM, INFO,
     $                     RES )
*
            IF( RES.NE.0 )
     $         RESULT = 1
*
            IF( INFO.NE.0 ) THEN
               WRITE( NOUT, FMT = 9998 )INFO
               RESULT = 1
            END IF
         END IF
*
*     Check to make sure that we have the right eigenvalues
*
         IF( LSAME( JOBZ, 'N' ) .OR. .NOT.LSAME( RANGE, 'A' ) ) THEN
*
*        Set up MYIL if necessary
*
            MYIL = IL
*
            IF( LSAME( RANGE, 'V' ) ) THEN
               MYIL = 1
*
               MINIL = 1
               MAXIL = N - M + 1
*
            ELSE
               IF( LSAME( RANGE, 'A' ) ) THEN
                  MYIL = 1
               END IF
               MINIL = MYIL
               MAXIL = MYIL
            END IF
*
*     Find the largest difference between the computed
*     and expected eigenvalues
*
            MINERROR = NORMWIN
*
            DO 160 MYIL = MINIL, MAXIL
               MAXERROR = 0
               DO 150 I = 1, M
                  ERROR = ABS( WIN( I+MYIL-1 )-WNEW( I+IPREPAD ) )
                  MAXERROR = MAX( MAXERROR, ERROR )
  150          CONTINUE
*
               MINERROR = MIN( MAXERROR, MINERROR )
  160       CONTINUE
*
            IF( MINERROR.GT.NORMWIN*FIVE*THRESH*EPS ) THEN
               WRITE( NOUT, FMT = 9997 )MINERROR, NORMWIN
               RESULT = 1
            END IF
         END IF
*
         IF( LSAME( JOBZ, 'V' ) .AND. LSAME( RANGE, 'A' ) ) THEN
            DO 170 I = 1, N
               WIN( I ) = WNEW( I+IPREPAD )
  170       CONTINUE
         END IF
      ELSE
         RESULT = 0
      END IF
*
*     All processes should report the same result
*
      CALL IGAMX2D( DESCA( 7 ), 'a', ' ', 1, 1, RESULT, 1, 1, 1, -1, -1,
     $              0 )
*
  180 CONTINUE
*
      RETURN
*
 9999 FORMAT( ' PDSYEVX returned INFO=', I7 )
 9998 FORMAT( ' PDSEPQTQ returned INFO=', I7 )
 9997 FORMAT( 'PDSEPTST2 minerror =', D10.2, ' normwin=', D10.2 )
      END
